﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Numerics;
using System.Windows.Forms;

namespace Climate_toolkit
{
    class CFourierAnalysisFactory
    {
        public static double[] get_dft_by_coefficients(double[] data, double delt, double T)
        {


            int N = data.Length;
            double Num = Convert.ToDouble(N);

            double[] y = new double[N];


            {
                for (int i = 0; i < N; i++)
                {
                    y[i] = data[i];
                }
            }


            double[] k = new double[N / 2];
            double[] index = new double[N];
            double Asum = 0;
            double Bsum = 0;

            double[] A = new double[N / 2];
            double[] B = new double[N / 2];
            double[] C = new double[N / 2];

            double[] spectral_power = new double[N / 2];

            // assign k's
            for (int i = 0; i < N / 2; i++)
            {
                if (i == 0)
                {
                    k[i] = 1;
                }
                else
                {
                    k[i] = k[i - 1] + 1;
                }
            }

            for (int i = 0; i < N; i++)
            {
                if (i == 0)
                {
                    index[i] = 1;
                }
                else
                {
                    index[i] = index[i - 1] + 1;
                }
            }

            // assign i index


            for (int j = 0; j < (N / 2); j++)
            {
                if (j != N / 2)
                {
                    for (int i = 0; i < Num; i++)
                    {
                        if (i == 0)
                        {
                            Asum = y[i] * Math.Cos(2 * Math.PI * k[j] * (index[i] * delt / T));
                            Bsum = y[i] * Math.Sin(2 * Math.PI * k[j] * (index[i] * delt / T));
                        }
                        else
                        {
                            Asum = Asum + y[i] * Math.Cos(2 * Math.PI * k[j] * (index[i] * delt / T));
                            Bsum = Bsum + y[i] * Math.Sin(2 * Math.PI * k[j] * (index[i] * delt / T));
                        }
                    }
                    A[j] = 2 / Num * Asum;
                    B[j] = 2 / Num * Bsum;
                }
                else // j (or "k") = N/2
                {
                    for (int i = 0; i < Num; i++)
                    {
                        if (i == 0)
                        {
                            Asum = y[i] * Math.Cos(Math.PI * Num * (delt / T));
                            Bsum = 0;
                        }
                        else
                        {
                            Asum = Asum + y[i] * Math.Cos(Math.PI * Num * (delt / T));
                            Bsum = 0;
                        }
                    }
                    A[j] = 1 / Num * Asum;
                    B[j] = 0;
                }
                C[j] = Math.Sqrt(Math.Pow(A[j], 2) + Math.Pow(B[j], 2));
                spectral_power[j] = Math.Pow(C[j], 2) / 2;

            }

            return spectral_power;

        }        
        
        public static double[] get_spectrum_scale_to_one(double[] y)
        {
            double[] scaled_spectrum = new double[y.Length];
            double sum = 0;
            for (int i = 0; i < y.Length; i++)
            {
                sum = sum + y[i];
            }

            for (int i = 0; i < y.Length; i++)
            {
                scaled_spectrum[i] = y[i] / sum;
            }
            return scaled_spectrum;
        }



        public static double get_spectrum_dof(double[] x, double[] spectrum, double fw)
        {
            double dof;
            dof = 2;

            return dof;
        }

        public static double[] get_theoretical_power_spectrum(double [] x, double delt, double[] spectrum)
        {
            // first get frequencies
            double[] omega = new double[spectrum.Length];
            double[] k = new double[spectrum.Length];

            double num = Convert.ToDouble(x.Length);

            // assign k's
            for (int i = 0; i < spectrum.Length; i++)
            {
                if (i == 0)
                {
                    k[i] = 1;
                }
                else
                {
                    k[i] = k[i - 1] + 1;
                }
            }

            for (int j = 0; j < spectrum.Length; j++)
            {
                omega[j] = 2 * Math.PI * k[j] / (num - 1);
            }

            // then get e-folding length T
            // get lag-1 autocorrelation a
            double[] a = new double[2];
            a = CStatsFactory.get_autocorrelation_function(x, 1);
            double T = -delt / Math.Log(a[1]);

            double[] iota = new double[spectrum.Length];

            for (int i = 0; i < spectrum.Length; i++)
            {
                iota[i] = (2 * T) / (1 + Math.Pow(T, 2) * Math.Pow(omega[i], 2));
            }
            return iota;



        }

        public static double[] get_sig_spectrum_chisquared(double[] x, double[] rednoisespectrum, int numsmoothingwindow, int numrealizations, double y, bool hanning)
        {
            double[] sig_spectrum = new double[rednoisespectrum.Length];
            int n; // degrees of freedom
            double v; // scaling for chi-squared test
            int M; // number of independent spectral estimates
            int N; // number of samples
            
            // numrealizations: number of time chunks

            int Mch; // number of time steps in time chunk
            int Msp; // number of spectral estimates

            N = x.Length;
            Mch = N / (numrealizations * numsmoothingwindow);
              //numrealizations is the time steps in the time chunk
            Msp = numrealizations;

            double fw = 1.0;
            if (hanning)
            {
                fw = 1.2; // from Hartmann notes
            }

            //int M = rednoisespectrum.Length * numsmoothingwindow * numrealizations;
            //v = x.Length / M; // could multiply by fw to account for smoothing
            //v = 2 * numsmoothingwindow*numrealizations;
            n = N / (Mch / 2);
            
            //n = N / Msp;
            v = n - 1;

            double critical_value = CHypothesisTestingFactory.get_Chisquared_sig_level(n, y);


            for (int i = 0; i < rednoisespectrum.Length; i++)
            {
                sig_spectrum[i] = (rednoisespectrum[i] * critical_value) / v;
            }

            return sig_spectrum;
        }

        public static double[] get_sig_spectrum_Fstatistic(double[] x, double[] rednoisespectrum, int numsmoothingwindow, int numrealizations, double y, bool hanning)
        {
            double[] sig_spectrum = new double[rednoisespectrum.Length];
            //double[] F = new double[spectrum.Length];

            int n; // degrees of freedom
            int v; // scaling factor
            int M; // number of independent spectral estimates
            int N; // number of samples

            // numrealizations: number of time chunks

            int Mch; // number of time steps in time chunk

            N = x.Length;
            Mch = N / (numrealizations * numsmoothingwindow); ;
            //numrealizations is the time steps in the time chunk

            double fw = 1.0;
            if (hanning)
            {
                fw = 1.2; // from Hartmann notes
            }

            //int M = rednoisespectrum.Length * numsmoothingwindow * numrealizations;
            //v = x.Length / M; // could multiply by fw to account for smoothing
            //v = 2 * numsmoothingwindow*numrealizations;
            n = N / (Mch / 2);
            v = n - 1;
            //v = n * fw;

            //int a = spectrum.Length -1; // dof series 1
            int a = v;
            //int b = rednoisespectrum.Length - 1; // dof series 2
            int b = 999999999; // infinity

            double critical_value = CHypothesisTestingFactory.get_Fstatistic_sig_level(a, b, y);

            for (int i = 0; i < rednoisespectrum.Length; i++)
            {
                sig_spectrum[i] = rednoisespectrum[i] * critical_value;
            }
            return sig_spectrum;
        }

        public static double[] get_boxcar_window(double[] x)
        {
            double[] boxcar = new double[x.Length];
            double T = x.Length;

            for (int i = 0; i < x.Length; i++)
            {
                boxcar[i] = x[i] / T;
            }

            return boxcar;
        }

        public static double[] get_hann_windowed_ts(double[] x)
        {
            double[] w = new double[x.Length];
            double[] hann = new double[x.Length];
            double T = x.Length;
            double t;
            double alpha = 0.5;
            for (int i = 0; i < x.Length; i++)
            {
                t = Convert.ToDouble(i);
                w[i] = alpha * (1 - alpha) * Math.Cos(Math.PI * t / T);
            }

            for (int i = 0; i < x.Length; i++)
            {
                hann[i] = w[i] * x[i];
            }


            return hann;

        }

        public static double[] get_hamming_windowed_ts(double[] x)
        {
            double[] w = new double[x.Length];
            double[] hamming = new double[x.Length];
            double T = x.Length;
            double t;
            double alpha = 0.54;
            for (int i = 0; i < x.Length; i++)
            {
                t = Convert.ToDouble(i);
                w[i] = alpha * (1 - alpha) * Math.Cos(Math.PI * t / T);
            }

            for (int i = 0; i < x.Length; i++)
            {
                hamming[i] = w[i] * x[i];
            }


            return hamming;

        }

        public static double[] get_parzen_windowed_ts(double[] x)
        {
            // special form from Hartmann notes
            double[] w = new double[x.Length];
            double[] parzen = new double[x.Length];
            double T = x.Length;
            double t;
            for (int i = 0; i < x.Length / 2; i++)
            {
                t = Convert.ToDouble(i);
                w[i] = 1 - 6 * Math.Pow(t / T, 2) + 6 * Math.Pow(Math.Abs(t) / T, 3);
            }

            for (int i = x.Length / 2; i < x.Length; i++)
            {
                t = Convert.ToDouble(i);
                w[i] = 2 * Math.Pow(1 - Math.Abs(t) / T, 3);
            }

            for (int i = 0; i < x.Length; i++)
            {
                parzen[i] = w[i] * x[i];
            }


            return parzen;

        }

        public static double[] get_dft_spectrum(double[] x, int n, double delt, double T, bool overlap)
        {
            // n = number of chucks to subdivide the time series
            if (n == 0)
            {
                n = n + 1;
            }
            int chunk_length = 0;
            double period_chunk = T;
            double chunk_length_int_test;
            // check for integer length of chunks // this needs to be moved ousideo of this method
            if (overlap)
            {
                chunk_length_int_test = 2 * x.Length / (n + 1);
            }
            else
            {
                chunk_length_int_test = x.Length / n;
            }

            if (chunk_length_int_test != Math.Round(chunk_length_int_test))
            {
                MessageBox.Show("Time chunks must have an integer length. Try a different number of time chunks");
                //Application.Exit();
            }
            else
            {
                chunk_length = Convert.ToInt32(chunk_length_int_test);
            }

            period_chunk = delt * (chunk_length - 1);
            //int n = 2*x.Length/chunk_length -1;
            double[,] ts_chunks = new double[n, chunk_length];
            double[] ts = new double[chunk_length];

            int chunk_start = 0;
            int chunk_end = chunk_start + chunk_length - 1;
            int count = 0;

            // check if overlapping is applied
            int overlap_factor = 1; // no overlap
            if (overlap)
            {
                overlap_factor = 2; // overlap by half chunk length
            }
            for (int i = 0; i < n; i++)
            {
                if (i == 0)
                {
                    chunk_start = 0;
                    chunk_end = chunk_start + chunk_length - 1;
                }
                else
                {
                    chunk_start = chunk_start + chunk_length / overlap_factor;
                    chunk_end = chunk_end + chunk_length / overlap_factor;
                }
                count = 0;
                for (int t = chunk_start; t <= chunk_end; t++)
                {
                    ts_chunks[i, count] = x[t];
                    count++;
                }
            }

            double[] spectrum = new double[chunk_length / 2]; // number of spectral estimates is N/2
            double[,] spectra = new double[n, chunk_length / 2];
            double[] wave_number_power = new double[n];
            for (int i = 0; i < n; i++)
            {
                for (int t = 0; t < chunk_length; t++)
                {
                    ts[t] = ts_chunks[i, t];
                }

                spectrum = get_dft_by_coefficients(ts, delt, period_chunk);
                for (int k = 0; k < chunk_length / 2; k++)
                {
                    spectra[i, k] = spectrum[k];
                }
            }

            double[] k_values = new double[n];
            double k_avg = 0;
            double[] spectra_avg = new double[chunk_length / 2]; // number of spectral estimates is N/2

            for (int k = 0; k < chunk_length / 2; k++) // number of spectral estimates is N/2
            {
                for (int i = 0; i < n; i++)
                {
                    k_values[i] = spectra[i, k];
                    k_avg = CStatsFactory.get_mean(k_values);
                }
                spectra_avg[k] = k_avg;
            }

            return spectra_avg;
        }
    }
}
