﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Climate_toolkit
{
    class CMEM_factory
    {
        public static double[] get_Burg_AR_coeff(double[] x, int m, out double sig2)
        {
            // modifed from code by Mike Dettinger mempwr.f for estimating MEM spectrum
            int n = x.Length;

            double[] a = new double[n];
            double[] efk = new double[n];
            double[] ebk = new double[n];
            double[] efk1 = new double[n];
            double[] ebk1 = new double[n];
            double[,] aa = new double[n, n];
            double[] rho = new double[n];
            double[] rho2 = new double[n];
            double sumn;
            double sumd;

            sig2 = 0.0;

            // Compute the estimate of the autocorrelation at lag 0
            double rho0 = 0;
            for (int i = 0; i < n; i++)
            {
                rho0 = rho0 + Math.Pow(Math.Abs(x[i]), 2) / n;
            }

            //Initialize the forward and backward prediction errors

            for (int i = 1; i < n; i++)
            {
                efk1[i] = x[i];
                ebk1[i - 1] = x[i - 1];
            }

            // begin recursion
            for (int k = 0; k < m; k++)
            {
                //compute the reflection coefficient estimate
                sumn = 0;
                sumd = 0;
                for (int i = k + 1; i < n; i++)
                {
                    sumn = sumn + efk1[i] * ebk1[i - 1];
                    sumd = sumd + Math.Pow(Math.Abs(efk1[i]), 2) + Math.Pow(Math.Abs(ebk1[i - 1]), 2);
                }
                aa[k, k] = -2 * sumn / sumd;

                // update the prediction error power
                if (k == 0)
                {
                    rho[k] = (1 - Math.Pow(Math.Abs(aa[k, k]), 2)) * rho0;
                }
                if (k > 0) 
                {
                    rho[k] = (1 - Math.Pow(Math.Abs(aa[k, k]), 2)) * rho[k - 1]; 
                }
                if (m > 1)
                {
                    if (k > 0)
                    {
                        // update the prediction error filter coefficients
                        for (int j = 0; j <= k - 1; j++)
                        {
                            aa[j, k] = aa[j, k - 1] + aa[k, k] * aa[k - j, k - 1];
                            double p = aa[j, k - 1];
                            double q = aa[k, k];
                            double r = aa[k - j - 1, k - 1];
                            aa[j, k] = p + q * r;
                        }
                    }
                    // update the foward and backward prediction errors
                    for (int i = k + 2; i < n; i++)
                    {
                        efk[i] = efk1[i] + aa[k, k] * ebk1[i - 1];
                        ebk[i - 1] = ebk1[i - 2] + aa[k, k] * efk1[i - 1];
                    }

                    for (int i = k + 2; i < n; i++)
                    {
                        efk1[i] = efk[i];
                        ebk1[i - 1] = ebk[i - 1];
                    }
                }
                sig2 = rho[m-1];
                for (int i = 0; i < m; i++)
                {
                    a[i] = -aa[i, m-1];
                }
            }
            return a;
        }

        public static double[] get_MEM_spectrum(double[] a, double[] f_delta, double sig2, double f_interval, int m, int nfreq)
        {
            // modifed from code by Mike Dettinger mempwr.f for estimating MEM spectrum
            double theta;

            double f_inc;

            double xr; // x real
            double xi; // x imaginary
            double cofreal;
            double cofimag;

            double sumreal;
            double sumimag;
            double coftemp;

            double[] power = new double[nfreq];
            //power[0] = double.NaN;

            //f_inc = f_interval/256.0;

            //for (int i = 0; i < nfreq; i++)
            //{
            //    if (i == 0)
            //    {
            //        f_delta[i] = f_inc * .5;
            //    }
            //    else
            //    {
            //        f_delta[i] = f_delta[i - 1] + f_inc * .5;
            //    }
            //}
            
            for (int j = 0; j < nfreq; j++)
            {
                
                // Separate the complex exponential into real and imaginary parts by Euler's formula
                theta = 2*Math.PI * f_delta[j];
                xr = Math.Cos(theta);
                xi = Math.Sin(theta);
                cofreal = 1.0;
                cofimag = 0.0;
                //  Form the sum (k=1,...,M) found in the denominator 

                sumreal = 1.0;
                sumimag = 0.0;
                for (int k = 0; k < m; k++)
                {
                    coftemp = cofreal;
                    cofreal = cofreal * xr - cofimag * xi;
                    cofimag = cofimag * xr + coftemp * xi;
                    sumreal = sumreal - a[k] * cofreal;
                    sumimag = sumimag - a[k] * cofimag;
                }

                // compute the power at the given frequency

                power[j] = sig2 / (Math.Pow(sumreal, 2) + Math.Pow(sumimag, 2));
            }
            return power;
        }

    }
}
