﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Data;

namespace Climate_toolkit
{
    public class CPreprocessDataFactory
    {

        private Cdatamaster datamaster;
        private string selecteddataseries; // this is only for the preprocessing tasks
        private string selecteddataseries_interpolated;

        public CPreprocessDataFactory(Cdatamaster datamasterToUse, string selecteddataseriesToUse)
        {
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;
        }

        public class CInterpolationFactory
        {

            public void interpolate_data(string selectedinterpmethod)
            {
                if (selectedinterpmethod == "Natural Cubic Spline")
                {
                    
                    
                }
                    

            }

            public static double[] get_intervals_for_interpolation(double start, double end, int numpoints)
            {

                double [] x = new double[numpoints];
                double range;
                double increment;
                double dnumpoints;

                dnumpoints = (double) numpoints;

                range = end - start;
                increment = range / (dnumpoints-1);


                
                for (int i = 0; i <= numpoints-1; i++)
                {
                    if (i == 0)
                    {
                        x[i] = start;
                    }
                    else
                    {
                        x[i] = x[i-1]+increment;
                    }
                }
                return x;
            }



            #region Interpolation Methods

            //public static double LinearInterpolation(double[] x, double[] y, double xval)
            //{
            //    double yval = 0.0;
            //    for (int i = 0; i < x.Length - 1; i++)
            //    {
            //        if (xval >= x[i] && xval < x[i + 1])
            //        {
            //            yval = y[i] + (xval - x[i]) * (y[i + 1] - y[i]) / (x[i + 1] - x[i]);
            //        }
            //    }
            //    return yval;
            //}

            public static double LagrangeInterpolation(double[] x, double[] y, double xval)
            {
                double yval = 0.0;
                double Products = y[0];
                for (int i = 0; i < x.Length; i++)
                {
                    Products = y[i];
                    for (int j = 0; j < x.Length; j++)
                    {
                        if (i != j)
                        {
                            Products *= (xval - x[j]) / (x[i] - x[j]);
                        }
                    }
                    yval += Products;
                }
                return yval;
            }

            public static double[] LagrangeInterpolation(double[] x, double[] y, double[] xvals)
            {
                double[] yvals = new double[xvals.Length];
                for (int i = 0; i < xvals.Length; i++)
                    yvals[i] = LagrangeInterpolation(x, y, xvals[i]);
                return yvals;
            }

            public static double BarycentricInterpolation(double[] x, double[] y, double xval)
            {
                double product;
                double deltaX;
                double bc1 = 0;
                double bc2 = 0;
                int size = x.Length;
                double[] weights = new double[size];

                for (int i = 0; i < size; i++)
                {
                    product = 1;
                    for (int j = 0; j < size; j++)
                    {
                        if (i != j)
                        {
                            product *= (x[i] - x[j]);
                            weights[i] = 1.0 / product;
                        }
                    }
                }

                for (int i = 0; i < size; i++)
                {
                    deltaX = weights[i] / (xval - x[i]);
                    bc1 += y[i] * deltaX;
                    bc2 += deltaX;
                }
                return bc1 / bc2;
            }

            public static double[] BarycentricInterpolation(double[] x, double[] y, double[] xvals)
            {
                double[] yvals = new double[xvals.Length];
                for (int i = 0; i < xvals.Length; i++)
                    yvals[i] = BarycentricInterpolation(x, y, xvals[i]);
                return yvals;
            }

            public static double NewtonDividedDifferenceInterpolation(double[] x, double[] y, double xval)
            {
                double yval;
                int size = x.Length;
                double[] tarray = new double[size];
                for (int i = 0; i < size; i++)
                {
                    tarray[i] = y[i];
                }

                for (int i = 0; i < size - 1; i++)
                {
                    for (int j = size - 1; j > i; j--)
                    {
                        tarray[j] = (tarray[j - 1] - tarray[j]) / (x[j - 1 - i] - x[j]);
                    }
                }

                yval = tarray[size - 1];

                for (int i = size - 2; i >= 0; i--)
                {
                    yval = tarray[i] + (xval - x[i]) * yval;
                }
                return yval;
            }

            public static double[] NewtonDividedDifferenceInterpolation(double[] x, double[] y, double[] xvals)
            {
                double[] yvals = new double[xvals.Length];
                for (int i = 0; i < xvals.Length; i++)
                    yvals[i] = NewtonDividedDifferenceInterpolation(x, y, xvals[i]);
                return yvals;
            }
            #endregion  

            #region Natural Cubic Splines

            public static double NaturalCubicSpline(double[] x1, double[] y1, double xvalue)
            {
                //int i, j, m;
                //double S = 0.0;
                //double delta = 0.0;

                //int n = x1.GetLength(0);
                 
                //double[] x = new double [n + 1];
                //double[] y = new double [n + 1];

                //for (int k = 0; k <= n - 1; k++)
                //{
                //    x[k] = x1[k];
                //    y[k] = y1[k];
                //}

                //x[n] = 0;
                //y[n] = 0;

                ////int n = 4; //max number of coefficients: A,B,C,D for cubic spline

                //double[] A = new double[n + 1];
                //double[] B = new double[n + 1];
                //double[] C = new double[n + 1];
                //double[] D = new double[n + 1];
                //double[] H = new double[n + 1];
                //double[] XA = new double[n + 1];
                //double[] XL = new double[n + 1];
                //double[] XU = new double[n + 1];
                //double[] XZ = new double[n + 1];

                //for (i = 0; i < n; i++) A[i] = y[i];

                //m = n - 1;
                //for (i = 0; i <= m; i++)
                //    H[i] = x[i + 1] - x[i];

                //for (i = 1; i <= m; i++)
                //    XA[i] = 3 * (A[i + 1] * H[i - 1] - A[i] * (x[i + 1] - x[i - 1]) + A[i - 1] * H[i]) / (H[i] * H[i - 1]);

                //XL[0] = 1; XU[0] = 0; XZ[0] = 0;
                //for (i = 1; i <= m; i++)
                //{
                //    XL[i] = 2 * (x[i + 1] - x[i - 1]) - H[i - 1] * XU[i - 1];
                //    XU[i] = H[i] / XL[i];
                //    XZ[i] = (XA[i] - H[i - 1] * XZ[i - 1]) / XL[i];
                //}
                //XL[n] = 1; XZ[n] = 0; C[n] = 0;

                //for (i = 0; i <= m; i++)
                //{
                //    j = m - i;
                //    C[j] = XZ[j] - XU[j] * C[j + 1];
                //    B[j] = (A[j + 1] - A[j]) / H[j] - H[j] * (C[j + 1] + 2 * C[j]) / 3;
                //    D[j] = (C[j + 1] - C[j]) / (3 * H[j]);
                //}

                //for (i = 0; i <= m; i++)
                //{
                //    if (xvalue >= x[i] && xvalue < x[i + 1])
                //    {
                //        delta = xvalue - x[i];
                //        S = A[i] + B[i] * delta + C[i] * delta * delta + D[i] * delta * delta * delta;
                //    }
                //}
                //return S;

                double v;
                alglib.spline1dinterpolant s;
                alglib.spline1dbuildcubic(x1, y1, out s);
                v = alglib.spline1dcalc(s, xvalue);
                return v;
            }

            #endregion

            #region Clamped Cubic Splines

            public static double ClampedCubicSpline(double[] x1, double[] y1, double fp0, double fpn, double xvalue)
            {
                int i, j, m;
                double S = 0.0;
                double delta = 0.0;

                int n = x1.GetLength(0);

                double[] x = new double[n + 1];
                double[] y = new double[n + 1];

                for (int k = 0; k <= n - 1; k++)
                {
                    x[k] = x1[k];
                    y[k] = y1[k];
                }

                x[n] = 0;
                y[n] = 0;


                //int n = 4; //max number of coefficients: A,B,C,D for cubic spline

                double[] A = new double[n + 1];
                double[] B = new double[n + 1];
                double[] C = new double[n + 1];
                double[] D = new double[n + 1];
                double[] H = new double[n + 1];
                double[] XA = new double[n + 1];
                double[] XL = new double[n + 1];
                double[] XU = new double[n + 1];
                double[] XZ = new double[n + 1];

                for (i = 0; i < n; i++) A[i] = y[i];

                m = n - 1;
                for (i = 0; i <= m; i++)
                    H[i] = x[i + 1] - x[i];

                XA[0] = 3 * (A[1] - A[0]) / H[0] - 3 * fp0;
                XA[n] = 3 * fpn - 3 * (A[n] - A[n - 1]) / H[n - 1];

                for (i = 1; i <= m; i++)
                    XA[i] = 3 * (A[i + 1] * H[i - 1] - A[i] * (x[i + 1] - x[i - 1]) + A[i - 1] * H[i]) / (H[i] * H[i - 1]);

                XL[0] = 2 * H[0]; XU[0] = 0.5; XZ[0] = XA[0] / XL[0];
                for (i = 1; i <= m; i++)
                {
                    XL[i] = 2 * (x[i + 1] - x[i - 1]) - H[i - 1] * XU[i - 1];
                    XU[i] = H[i] / XL[i];
                    XZ[i] = (XA[i] - H[i - 1] * XZ[i - 1]) / XL[i];
                }
                XL[n] = H[n - 1] * (2 - XU[n - 1]);
                XZ[n] = (XA[n] - H[n - 1] * XZ[n - 1]) / XL[n];
                C[n] = XZ[n];

                for (i = 1; i <= n; i++)
                {
                    j = n - i;
                    C[j] = XZ[j] - XU[j] * C[j + 1];
                    B[j] = (A[j + 1] - A[j]) / H[j] - H[j] * (C[j + 1] + 2 * C[j]) / 3;
                    D[j] = (C[j + 1] - C[j]) / (3 * H[j]);
                }

                Console.WriteLine("\n\nTesting clamped cubic splines");
                Console.WriteLine("Input data stored in (x[i],y[i]). Value to interpolate = {0}\n", xvalue);
                Console.WriteLine("i\t" + "x[i]\t" + "y[i]\t" + "A[i]\t" + "B[i]\t" + "C[i]\t" + "D[i]\n");
                for (i = 0; i < n; i++)
                {
                    Console.WriteLine(i.ToString() + "\t" + x[i].ToString("0.0000") + "\t" + y[i].ToString("0.0000") + "\t" + A[i].ToString("0.0000") + "\t" + B[i].ToString("0.0000") + "\t" + C[i].ToString("0.0000") + "\t" + D[i].ToString("0.0000") + "\n");
                }

                for (i = 0; i <= m; i++)
                {
                    if (xvalue >= x[i] && xvalue < x[i + 1])
                    {
                        delta = xvalue - x[i];
                        S = A[i] + B[i] * delta + C[i] * delta * delta + D[i] * delta * delta * delta;
                        Console.WriteLine("Interpolated x value = y({0}) = {1}\n", xvalue, S);
                    }
                }
                return S;
            }

            public static double AkimaSpline(double[] x1, double[] y1, double xvalue)
            {
                double v;
                alglib.spline1dinterpolant s;
                alglib.spline1dbuildakima(x1, y1, out s);
                v = alglib.spline1dcalc(s, xvalue);
                return v;
            }

            public static double LinearInterpolation(double[] x1, double[] y1, double xvalue)
            {
                double v;
                alglib.spline1dinterpolant s;
                alglib.spline1dbuildlinear(x1, y1, out s);
                v = alglib.spline1dcalc(s, xvalue);
                return v;
            }

            //public static double HermiteSpline(double[] x1, double[] y1, double xvalue)
            //{
            //    double v;
            //    alglib.spline1dinterpolant s;
            //    alglib.spline1dbuildhermite(x1, y1, 
            //    v = alglib.spline1dcalc(s, xvalue);
            //    return v;
            //}





            #endregion


 
        }
        public class CCumulativeDepatureFactory
        {
            public static double[] getCumulativeDeparture(double[] x, out double mean)
            {
                double departure = 0;
                double[] cumulative_departure = new double[x.GetLength(0)];

                int n = x.GetLength(0);
                mean = CStatsFactory.get_mean(x);

                for (int i = 0; i < n; i++)
                {
                    departure = x[i] - mean;
                    if (i == 0)
                    {
                        cumulative_departure[i] = departure;
                    }
                    else
                    {
                        cumulative_departure[i] = cumulative_departure[i-1] + departure;
                    }
                }
                return cumulative_departure;
            }



        }

        public class CDetrendFactory
        {
            public static double get_trend(double[] x, double [] y, int m, double t)
            {
                double[] linear_trend;
                //int m = 2;
                //double t = 1950;
                int info;
                alglib.barycentricinterpolant p;
                alglib.polynomialfitreport rep;
                double v;

                alglib.polynomialfit(x, y, m, out info, out p, out rep);
                v = alglib.barycentriccalc(p, t);

                return v;


            }
            public static void get_differencing(double[] x, double[] y, int order, out double[] residuals, out double[] xnew)
            {
                int n = x.Length;
                residuals = new double[n - order];
                double[] trend = new double[n - order];
                xnew = new double[n - order];
                double[] temp1;
                double[] temp2;
                int count = 1;
                temp1 = new double[n];

                for (int k = 0; k < order; k++)
                {
                    temp2 = new double[n - count]; 
                    if(k == 0) temp1 = y;

                    for (int i = 0; i < n-count; i++)
                    {
                        temp2[i] = temp1[i + 1] - temp1[i];
                    }
                    temp1 = new double[n - count];
                    temp1 = temp2;

                    count = count + 1;
                    residuals = temp2;
                   
                }

                for (int i = 0; i < n - order; i++)
                {
                    xnew[i] = x[i + order];
                }
                //return differenced;
            }
        }

        
    }
}
