﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
//using MathNet.Numerics.Distributions;
//using MathNet.Numerics.LinearAlgebra.Double;

namespace Climate_toolkit
{
    class CProjectionFactory
    {
        private Cdatamaster datamaster;
        private string selecteddataseries; // this is only for the preprocessing tasks

        public CProjectionFactory(Cdatamaster datamasterToUse, string selecteddataseriesToUse)
        {
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;
        }

        public static void identify_AR_model(double[] x, int minorder, int maxorder, out double[] AIC, out int order, out double[,] sim, out double[,] residuals, out double[][] AR_coeff)
        {
            int nummodels = maxorder - minorder + 1;
            AIC = new double[nummodels];
            order = 1;

            double[] a = new double[maxorder];
            double[] res = new double[x.Length];

            double lowestAIC = 0;
            double [] ARsim = new double[x.Length];

            sim = new double[nummodels, x.Length];
            residuals = new double[nummodels, x.Length];
            //AR_coeff = new double[nummodels, maxorder];

            AR_coeff = new double[nummodels][];

            for (int i = 0; i < nummodels; i++)
            {
                AR_coeff[i] = new double[i + 1];
            }

            //double order = 1;
            if (minorder < 1)
            {
                return;
            }

            for (int i = minorder; i <= maxorder; i++)
            {
                // get AR coefficients and the AIC
                double[] r = CStatsFactory.get_autocorrelation_function(x, x.Length + 1);
                //double[] a = get_yulewalker(r, i);
                a = get_yulewalker(r, i);

                get_ARsim_residuals(x, a, i, out ARsim, out res);

                for (int j = 0; j < i; j++)
                {
                    AR_coeff[i - 1][j] = a[j];
                }

                for (int j = 0; j < x.Length; j++)
                {
                    sim[i - 1, j] = ARsim[j];
                    residuals[i - 1, j] = res[j];
                }

                AIC[i - 1] = get_AIC(x, res, i);

                if (i == 1)
                {
                    lowestAIC = Math.Abs(AIC[i - 1]);
                }

                else
                {
                    if (Math.Abs(AIC[i - 1]) < Math.Abs(lowestAIC))
                    {
                        lowestAIC = AIC[i - 1];
                        order = i;
                    }
                }
            }
        }

        public static void get_single_AR_model(double[] x, int order, out double[] AIC, out double[,] sim, out double[,] residuals, out double[][] AR_coeff)
        {
            AR_coeff = new double[1][];
           
            //for (int i = 0; i < order; i++)
            //{
                AR_coeff[0] = new double[order];
            //}

            AIC = new double[1];
            double[] a = new double[order];
            double[] res = new double[x.Length];

            double[] ARsim = new double[x.Length];

            sim = new double[1,x.Length];
            residuals = new double[1,x.Length];

            // get AR coefficients and the AIC
            double[] r = CStatsFactory.get_autocorrelation_function(x, x.Length + 1);
            a = get_yulewalker(r, order);

            get_ARsim_residuals(x, a, order, out ARsim, out res);

            for (int j = 0; j < order; j++)
            {
                AR_coeff[0][j] = a[j];
            }

            for (int j = 0; j < x.Length; j++)
            {
                sim[0,j] = ARsim[j];
                residuals[0,j] = res[j];
            }

            AIC[0] = get_AIC(x, res, order);
        }

        public static double[] get_yulewalker(double[] r, int k)
        {
            // r is a vector of autocorrelations, including 1 at lag 0
            // k is the order of the AR model

            if (r.GetLength(0) <= 1)
            {

            }

            double[] rtrim = new double[k];

            for (int i = 0; i < k; i++)
            {
                rtrim[i] = r[i + 1]; // skip the first at lag 0
            }
            //var rvar = new DenseVector(rtrim);
            
            double[,] zeros = new double[k, k];

            //for (int i = 0; i < k; i++)
            //{
            //    for (int j = 0; j < k; j++)
            //    {
            //        zeros[i, j] = 0;
            //    }
            //}

            //var Rvar = new DenseMatrix(zeros);
            double[,] R = new double[k, k];
            for (int i = 0; i < k; i++)
            {
                for (int j = 0; j < k; j++)
                {
                    R[i,j] = r[Math.Abs(i-j)];
                    //Rvar[i, j] = r[Math.Abs(i - j)];
                }
            }

            //var avar = Rvar.Inverse().LeftMultiply(rvar);
            //var Rvarinv = Rvar.Inverse();
            //double[] a = avar.ToArray();
            double[] a = new double[k];
            double [,] Rinv = new double[k,k];

            int info;
            alglib.matinvreport rep;
            alglib.ablas.rmatrixcopy(k, k, R, 0, 0, ref Rinv, 0, 0);
            alglib.rmatrixinverse(ref Rinv, out info, out rep);

            alglib.ablas.rmatrixmv(k, k, Rinv, 0, 0, 1, rtrim, 0, ref a, 0);

            return a;
        }

        public static double[,] inverse(double[,] a)
        {
            if (a.GetLength(0) != a.GetLength(1))
                throw new ArgumentException("inverse: matrix must be square");
            double[,] output = new double[a.GetLength(0), a.GetLength(1)];
            alglib.matinv.matinvreport rep = new alglib.matinv.matinvreport();
            int info = new int();
            alglib.matinv.rmatrixinverse(ref output, output.GetLength(0), ref info, rep);
            return output;
        }


        public static void get_ARsim_residuals(double[] y, double[] a, int order, out double[] yhat, out double[] yres)
        {
            // a.Length must equal order;
            yhat = new double[y.Length];
            int nyhat = y.Length - order;
            yres = new double[y.Length];

            double ar_sum;
            // assign zeros to the beginning of yhat because these cannot be simulated
            for (int i = 0; i < order; i++)
            {
                yhat[i] = 0;
            }
            //simulate yhat[i] using an ar model
            for (int t = order; t < y.Length; t++)
            {
                ar_sum = 0;
                for (int j = 0; j < order; j++) // j loops along the ar coefficients.
                {
                    ar_sum = ar_sum + a[j] * y[t - j - 1];
                }
                yhat[t] = ar_sum;
                yres[t] = y[t] - yhat[t];
            }
            //return yres;


        }
        public static double get_AIC(double[] y, double[] residuals, int order)
        {
            int N = y.Length;

            // get the variance of the residuals without NaN or 0 values
            double[] residuals_trim = new double[residuals.Length - order];
            for (int i = 0; i < N - order; i++)
            {
                residuals_trim[i] = residuals[i + order];
            }
            double residuals_trim_mean = CStatsFactory.get_mean(residuals_trim);
            double variance = CStatsFactory.get_variance(residuals_trim, residuals_trim_mean);
            double Ndouble = Convert.ToDouble(N);
            double orderdouble = Convert.ToDouble(order);

            //double AIC = Math.Log(variance * (1 + ((2 * orderdouble) / Ndouble)));
            double AIC = Ndouble * (Math.Log(variance) + 1) + 2 * (orderdouble + 1);
            double AICc = Ndouble * (Math.Log(variance) + 1) + (2 * (orderdouble + 1)) * (Ndouble / (Ndouble - orderdouble - 2));
            return AICc;
        }

        public static double[] get_ARforecast(double[] x, double[] a, int order, int nfore, double [] residuals)
        {
            int nx = x.Length;
            int na = a.Length;
            double ar_sum = 0;
            double[] rand = new double[nfore];

            // get only the nonzero residuals to account for the ar order
            int num_res = residuals.Length - order;
            double[] nonzero_residuals = new double[num_res];
            for (int i = 0; i < num_res; i++)
            {
                nonzero_residuals[i] = residuals[i + order];
            }

            rand = get_random_shock(nonzero_residuals, nfore);
            // make a vector that contains the time series and simulated values 
            double[] y = new double[nx + nfore];
            for (int t = 0; t < nx; t++)
            {
                y[t] = x[t];
            }

            for (int t = nx; t < nx + nfore; t++)
            {
                ar_sum = 0;
                for (int j = 0; j < order; j++) // j loops along the autocorrelation function. Start at a[1] since a[0]=1
                {
                    ar_sum = ar_sum + a[j] * y[t - j - 1];
                }
                y[t] = ar_sum + rand[t - nx];
            }
            return y;
        }

        public static double[] get_random_shock(double[] x, int n)
        {
            double[] randomnumbers = new double[n];



            double mean = CStatsFactory.get_mean(x);
            double var = CStatsFactory.get_variance(x, mean);
            double stdev = Math.Sqrt(var);
            // Wait to allow the timer to advance.
            //Thread.Sleep(1);
            //Normal norm = new Normal(mean, stdev);



            for (int i = 0; i < n; i++)
            {

                //randomnumbers[i] = AutoSeedRandoms_norm(mean, stdev);
                randomnumbers[i] = get_random_normal(mean, stdev);
                    
            }

            return randomnumbers;
        }

        public static int get_random_int(int min, int max)
        {
            int randomint = 0;
            //Thread.Sleep(10);
            //Random rand = new Random();
            //randomint = rand.Next(min, max);

            double y = 0;
            try
            {

                alglib.hqrnd.hqrndstate state = new alglib.hqrnd.hqrndstate();

                //alglib.hqrnd rand = new alglib.hqrnd();

                alglib.hqrnd.hqrndrandomize(state);
                randomint = alglib.hqrnd.hqrnduniformi(state, max);
 
            }
            catch { };


            return randomint;
        }

        // Create a random object with a timer-generated seed.

        public static double get_random_normal(double mean, double stdev)
        {
            double y = 0;
            try
            {

                alglib.hqrnd.hqrndstate state = new alglib.hqrnd.hqrndstate();

                //alglib.hqrnd rand = new alglib.hqrnd();

                alglib.hqrnd.hqrndrandomize(state);
                y = alglib.hqrnd.hqrndnormal(state);
                y = CStatsFactory.CStandardizeFactory.get_inverse_z(y, mean, stdev);
            }
            catch { };
            return y;
        }
        //public static double AutoSeedRandoms_norm(double mean, double stdev)
        //{
        //    // Wait to allow the timer to advance.
        //    //Thread.Sleep(1);
        //    Normal autoRand = new Normal(mean, stdev);

        //    double x = RunDoubleRandoms(autoRand);
            
        //    return x;
        //}

        // Create a random object with a timer-generated seed.
        public static int AutoSeedRandoms_int(int min, int max)
        {
            // Wait to allow the timer to advance.
            Thread.Sleep(1);

            Random autoRand = new Random();

            int x = RunIntRandoms(autoRand, min, max);
            return x;
        }

        static int RunIntRandoms(Random autoRand, int min, int max)
        {
            int x = autoRand.Next(min, max+1);
            //Console.WriteLine(x.ToString());

            //Console.ReadLine();
            return x;
        }

        //static double RunDoubleRandoms(Normal autoRand)
        //{
        //    Thread.Sleep(2);
        //    double x = autoRand.Sample();
        //    return x;
        //}


    }
}
