﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Climate_toolkit
{
    class CRegressionFactory
    {
        public static double get_a1_onepredictor(double[] x, double[] y)
        {
            double xmean = CStatsFactory.get_mean(x);
            double ymean = CStatsFactory.get_mean(y);

            double cov_xy = CStatsFactory.get_covariance(x, y, xmean, ymean);

            // get denominator term

            double xvar = CStatsFactory.get_variance(x, xmean);

            double a1 = cov_xy / xvar;
            return a1;
        }

        public static double get_a0_onepredictor(double xmean, double ymean, double a1)
        {
            double a0 = ymean - a1 * xmean;
            return a0;
        }
        public static void get_regression_line_onepredictor(double[] x, double a0, double a1, out double xmin, out double xmax, out double y_xmin, out double y_xmax)
        {
            xmin = x.Min();
            xmax = x.Max();
            y_xmin = xmin * a1 + a0;
            y_xmax = xmax * a1 + a0;
        }
        public static double get_corr_coeff(double[] x, double[] y)
        {
            double xmean = CStatsFactory.get_mean(x);
            double ymean = CStatsFactory.get_mean(y);

            double xy_cov = CStatsFactory.get_covariance(x, y, xmean, ymean);
            double xstdev = CStatsFactory.get_stdev(x, xmean);
            double ystdev = CStatsFactory.get_stdev(y, ymean);
            double r = xy_cov / (xstdev * ystdev);
            return r;
        }

        public static void get_best_lag(double[] x, double[] y, int max_positive_lag, int max_negative_lag, out double r_max_positive, out int best_lag_positive, out double r_max_negative, out int best_lag_negative, out double[] rvalues, out int[] lags)
        {
            r_max_positive = -10;
            r_max_negative = 10;

            best_lag_positive = 0;
            best_lag_negative = 0;
            //camp = 0;

            double xm; // x mean
            double ym; // y mean
            double xs; // x variance
            double ys; // y variance
            double cl;
            int n;
            int ib;
            int ie;
            int ii = x.Length;

            lags = new int[x.Length+1];
            double r;
            rvalues = new double[x.Length+1];

            //int lagmx = 60;
            int lagmax = x.Length / 2;
            
            //lamp = -lagmax - 1; // lag amplitude?
            best_lag_positive = -lagmax - 1;  // lag max?
            int lagcount = 0;

            //for (int ilag = -lagmax; ilag <= lagmax; ilag ++)
            for (int ilag = -max_negative_lag; ilag <= max_positive_lag; ilag++)
            {
                lags[lagcount] = ilag;
                
                xm = 0;
                ym = 0;
                xs = 0;
                ys = 0;
                cl = 0;
                n = 0;



                ib = 0;  // i begin
                ie = ii; // i end

                if (ib + ilag < 1) ib = -ilag; // 1 - ilag;
                if (ie + ilag > ii) ie = ii - ilag;

                if (ib + 3 < ie) // ensures that the overlap of the time series is at least 3
                {
                    for (int i = ib; i < ie; i++)
                    {
                        xm = xm + x[i];
                        ym = ym + y[i + ilag];
                        n = n + 1;
                    }
                    xm = xm / n; // mean x
                    ym = ym / n; // mean y

                    for (int i = ib; i < ie; i++)
                    {
                        xs = xs + Math.Pow(x[i]-xm, 2) / n; // variance x
                        ys = ys + Math.Pow(y[i + ilag] - ym, 2) / n; // variance y
                        cl = cl + (x[i] - xm) * (y[i + ilag] - ym) / n; // covariance(xy)
                    }

                    r = cl / Math.Sqrt(xs * ys);
                    rvalues[lagcount] = r;

                    if (r > r_max_positive)
                    {
                        r_max_positive = r;
                        best_lag_positive = ilag;
                    }

                    if (r < r_max_negative)
                    {
                        r_max_negative = r;
                        best_lag_negative = ilag;
                    }
                }
                lagcount++;
            }
        }

        public static int get_correlation_coefficient_sample_size(double[] x, double[] y)
        {
            int N_adjusted;
            double N_adjusted_double;
            int N;
            double N_double;
            double[] r1x;
            double[] r1y;

            r1x = CStatsFactory.get_autocorrelation_function(x, 1);
            r1y = CStatsFactory.get_autocorrelation_function(y, 1);
            
            N = x.Length;
            N_double = Convert.ToDouble(N);
            N_adjusted_double = N_double * (1 - r1x[1] * r1y[1]) / (1 + r1x[1] * r1y[1]);
            N_adjusted_double = Math.Floor(N_adjusted_double);
            N_adjusted = Convert.ToInt32(N_adjusted_double);
            
            return N_adjusted;
        }

    }
}
