﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
//using MathNet.Numerics.LinearAlgebra.Double;
//using MathNet.Numerics.LinearAlgebra.Double.Factorization;

namespace Climate_toolkit
{
    class CSSAFactory
    {
        public static double[,] prepare_trajectory_matrix(double[] x, int M)
        {
            int N = x.Length;

            double[,] X = new double[M, N - M + 1];
            //double[,] X = new double[N - M + 1, M];
            for (int m = 0; m < M; m++)
            {
                for (int n = 0; n < N - M + 1; n++)
                {
                    // this makes a lag M by time N                    
                    X[m, n] = x[n + m];
                    // this makes a time M by lag N  
                    //X[n, m] = x[n + m];
                }
            }
            return X;


        }

        public static double [,] get_BK_covariance_matrix(double [,] X)
        {
            // get covariance matrix ATA because nrows > ncols
            int M = X.GetLength(0);
            int N = X.GetLength(1);
            double Ndouble = Convert.ToDouble(N);
            //int M = X.GetLength(1);
            //int N = X.GetLength(0);
            double[,] XT = new double[N, M];
            alglib.ablas.rmatrixtranspose(M, N, X, 0, 0, ref XT, 0, 0); 
                
            //var Xvar = new DenseMatrix(X);
            //var XvarT = Xvar.Transpose();

            double[,] C = new double[M,M];
            alglib.ablas.rmatrixgemm(M, M, N, 1 / Ndouble, X, 0, 0, 0, XT, 0, 0, 0, 0, ref C, 0, 0);

            //double[,] Cd = Xvar.Multiply(XvarT).ToArray();

            //for (int i = 0; i < M; i++)
            //{
            //    for (int j = 0; j < M; j++)
            //    {
            //        Cd[i, j] = Cd[i, j] / (N);
            //    }
            //}
            //return Cd;
            return C;
        }





        public static double[,] get_reconstructed_components(double[,] eofs, double[,] pcs)
        {
            // n is the number of the component for reconstruction 

            int M = eofs.GetLength(0); // length of embedding dimension
            int N = pcs.GetLength(0); // Ndata-M+1 of Broomhead and King
            int Ndata = pcs.GetLength(0) + M - 1; // length of the original time series

            double[,] q = new double[Ndata, M]; // array for the rc

            // get all RCs n
            try
            {
                for (int n = 0; n < M; n++) // total of M RCs available
                {
                    for (int i = 0; i < M - 1; i++)
                    {
                        q[i, n] = 0;


                        for (int j = 0; j <= i; j++)
                        {
                            q[i, n] = q[i, n] + pcs[i - j, n] * eofs[j, n];
                        }
                        q[i, n] = q[i, n] / (i + 1);
                    }

                    for (int i = M - 1; i < N; i++)
                    {
                        q[i, n] = 0;
                        for (int j = 0; j < M; j++)
                        {
                            q[i, n] = q[i, n] + pcs[i - j, n] * eofs[j, n];
                        }
                        q[i, n] = q[i, n] / M;
                    }

                    for (int i = N; i < Ndata; i++)
                    {
                        q[i, n] = 0;
                        for (int j = i - Ndata + M; j < M; j++)
                        {
                            q[i, n] = q[i, n] + pcs[i - j, n] * eofs[j, n];
                        }
                        q[i, n] = q[i, n] / (Ndata - i + 2);
                    }
                }
            }
            catch
            {
            }
            return q;
        }



    }
}
