﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Climate_toolkit
{
    class CStatsFactory
    {
        public class CStandardizeFactory
        {
            public static double[] get_standardize_z(double[] x, out double mean, out double variance)
            {
                double stdev;
                double[] z = new double[x.Length];
                mean = get_mean(x);
                variance = get_variance(x, mean);
                stdev = get_stdev(x, mean);
                
                for (int i = 0; i < x.Length; i++)
                {
                    z[i] = (x[i] - mean) / stdev;
                }
                return z;
            }

            public static double get_inverse_z(double z, double mean, double stdev)
            {
                double x = z * stdev + mean;
                return x;
            }

            public static double get_z_statistic(double x, double mean, double stdev)
            {
                double z = (x - mean) / stdev;
                return z;
            }
        }
        #region Gamma methods
        public class CGammaMethodsFactory
        {
            public static double[] get_gamma_Z(double [] x, out double x_mean, out double x_variance, out double alpha, out double beta)
            {
                double[] Z_gamma = new double[x.Length]; // standard normal random variable  

                double[] G = new double[x.GetLength(0)];
                double[] H = new double[x.GetLength(0)];
                
                // adjust cumulative probability to account for zero values
                double r; //probably of a zero value = (num zeros)/n
                double t = 0; // variable for converting cumulative probability to standard normal random Z from Abramowitz and Stegun (1965)

                // count zeros
                int count = 0;
                for (int i = 0; i < x.Length; i++)
                {
                    if (x[i] == 0)
                    {
                        count = count + 1;
                    }
                }
                r = count / x.Length;

                x_mean = get_mean(x);
                x_variance = get_variance(x, x_mean);
                double thomD = get_ThomD(x,x_mean);
                alpha = get_gamma_alpha(thomD);
                beta = get_gamma_beta(alpha, x_mean);
                
                for (int i = 0; i < x.GetLength(0); i++)
                {
                    G[i] = SpecialFunction.igam(alpha, x[i] / beta); // incomplete gamma function
                    H[i] = r + (1 - r) * G[i];
                }
                
                // coefficients
                double c0 = 2.515517;
                double c1 = 0.802853;
                double c2 = 0.010328;
                double d1 = 1.432788;
                double d2 = 0.189269;
                double d3 = 0.001308;
                
                for (int i = 0; i < x.Length; i++)
                {
                    if (H[i] > 0 && H[i] <= 0.5)
                    {
                        t = Math.Sqrt(Math.Log(1 / Math.Pow(H[i], 2), Math.E));
                        Z_gamma[i] = -(t - (c0 + c1 * t + c2 * Math.Pow(t, 2)) / (1 + d1 * t + d2 * Math.Pow(t, 2) + d3 * Math.Pow(t, 3)));

                    }
                    if (H[i] > 0.5 && H[i] < 1.0)
                    {
                        t = Math.Sqrt(Math.Log(1 / Math.Pow(1.0 - H[i], 2), Math.E));
                        Z_gamma[i] = +(t - (c0 + c1 * t + c2 * Math.Pow(t, 2)) / (1 + d1 * t + d2 * Math.Pow(t, 2) + d3 * Math.Pow(t, 3)));
                    }
                }
                return Z_gamma;

            }
            
            public static double[] get_gamma_distribution(double[] x, double[] y)
            {
                // get alpha and beta values
                double mean = get_mean(y);

                double thomD = get_ThomD(y, mean);
                
                double a = get_gamma_alpha(thomD);
                double b = get_gamma_beta(a, mean);
                
                
                
                double Y = Gamma(a);

                double[] g = new double[x.Length];

                for (int i = 0; i < x.Length; i++)
                {
                    g[i] = (Math.Pow(x[i] / b, a - 1) * Math.Exp(-x[i] / b)) / (b * Y);
                }
                return g;
            }



            public static double GammaLn(double x)
            //from Dos Passos
            {
                if (x <= 0) throw
                new Exception("Input value must be > 0");

                double[] coef = new double[14]
            {57.1562356658629235,
            -59.5979603554754912,
            14.1360979747417471,
            -0.491913816097620199,
             0.339946499848118887E-4,
             0.465236289270485756E-4,
            -0.983744753048795646E-4,
             0.158088703224912494E-3,
            -0.210264441724104883E-3,
             0.217439618115212643E-3,
            -0.164318106536763890E-3,
             0.844182239838527433E-4,
            -0.261908384015814087E-4,
             0.368991826595316234E-5};

                double denominator = x;
                double series = 0.999999999999997092;
                double temp = x + 5.24218750000000000;
                temp = (x + 0.5) * Math.Log(temp, Math.E) - temp;
                for (int j = 0; j < 14; j++)
                    series += coef[j] / ++denominator;
                return (temp + Math.Log(2.5066282746310005 * series / x));
            }


            public static double Gamma(double x)
            {
                if (x <= 0) throw
                    new Exception("Input value must be > 0");
                return Math.Exp(GammaLn(x));
            }
            
            public static double get_gamma_alpha(double A)
            {
                double alpha;

                alpha = (1 + Math.Sqrt(1 + (4 * A) / 3)) / (4 * A);

                return alpha;
            }

            public static double get_gamma_beta(double alpha, double mean)
            {
                double beta;

                beta = mean / alpha;

                return beta;
            }

            public static double get_ThomD(double[] x, double mean)
            {
                double A;
                int n = x.Length;
                double sum = 0;

                for (int i = 0; i < n; i++)
                {
                    if (x[i] > 0)
                    {
                        sum = sum + Math.Log(x[i], Math.E);
                    }
                }

                A = Math.Log(mean, Math.E) - sum / n;

                return A;
            }
        }
        #endregion Gamma methods

        public static double get_mean(double[] x)
        {
            double mean;
            int n = x.GetLength(0);
            double sum = 0;
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + x[i];
            }

            mean = sum / n;
            return mean;
        }

        public static double get_variance(double[] x, double mean)
        {
            double var;
            double sum = 0;
            int n = x.GetLength(0);
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + (x[i] - mean) * (x[i] - mean);
            }
            var = sum / num;

            return var;
        }

        public static double get_stdev(double[] x, double mean)
        {
            double var;
            double sum = 0;
            double stdev;
            int n = x.GetLength(0);
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + (x[i] - mean) * (x[i] - mean);
            }
            var = sum / num;
            stdev = Math.Sqrt(var);
            return stdev;
        }

        public static double get_skew(double[] x, double mean)
        {
            double var;
            double sum = 0;
            int n = x.GetLength(0);
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + Math.Pow((x[i] - mean),3);
            }
            var = sum / num;

            return var;
        }

        public static double get_kurtosis(double[] x, double mean)
        {
            double var;
            double sum = 0;
            int n = x.GetLength(0);
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + Math.Pow((x[i] - mean), 4);
            }
            var = sum / num;

            return var;
        }

        public static double get_covariance(double[] x, double[] y, double meanx, double meany)
        {
            double covar;
            double sum = 0;
            
            int n = x.GetLength(0);
            double num = Convert.ToDouble(n);

            for (int i = 0; i < n; i++)
            {
                sum = sum + (x[i] - meanx) * (y[i] - meany);
            }
            covar = sum / num;

            return covar;
        }        
   
        public static double [] get_histogram(double [] x, int nbin, double start, double end)
        {
            double[] hist = new double[nbin];
            double bininterval;
            double numbin = Convert.ToDouble(nbin);
            double [] bounds = new double[nbin+1];

            // first make a vector of thresholds for bin values
            bininterval = (end - start) / numbin;
            int ndata = x.GetLength(0); 

            for (int i = 0; i <= nbin; i++ )
            {
                if (i == 0)
                {
                    bounds[i] = start;
                }

                else
                {
                    bounds[i] = bounds[i - 1] + bininterval;
                }
            }

            for (int j = 0; j < ndata; j++)
            {
                for (int i = 0; i < nbin; i++)
                {
                    if (x[j] >= bounds[i] && x[j] < bounds[i + 1])
                    {
                        hist[i]++;
                    }
                    if (i == nbin - 1 && x[j] >= bounds[i + 1])
                    {
                        hist[i]++; // add another count if greater than the last nbin
                    }

                }
            }
            return hist;

        }


        public static double[] get_autocorrelation_function(double[] x, int maxlag)
        {
            double xmean = get_mean(x);
            int n = x.Length;
            double n_double = Convert.ToDouble(n);
            double[] autocov = new double[maxlag+1];
            double[] autocorr = new double[maxlag+1];
            for (int h = 0; h <= maxlag; h++)
            {
                double sum = 0;
                for (int i = 0; i < n - Math.Abs(h); i++)
                {
                    sum = sum + (x[i + Math.Abs(h)] - xmean) * (x[i] - xmean);
                }
                autocov[h] = (1 / (n_double-h)) * sum;
                autocorr[h] = autocov[h] / autocov[0];
            }

            return autocorr;
        }


        public static double[] get_moving_average(double[] x, int n)
        {
            int ma_length = x.Length - n + 1;
            double[] moving_average = new double[ma_length];

            double[] window = new double[n];
            int start_value = (n - 1) / 2;
            int center_value = 0;
            for (int i = 0; i < ma_length; i++)
            {
                if (i == 0)
                {
                    center_value = (n - 1) / 2;
                }
                for (int j = 0; j < n; j++)
                {
                    window[j] = x[center_value - (n - 1) / 2 + j];
                }
                moving_average[i] = CStatsFactory.get_mean(window);
                center_value++;
            }


            return moving_average;
        }

        public static double get_corr_coeff_tstatistic(double r, double N)
        {
            double t = r * Math.Pow(N - 2, 0.5) / Math.Pow(1 - Math.Pow(r, 2), 0.5);
            return t;
        }

        public static double[] get_normal_distribution(double[] x, double[] y)
        {
            double mean = get_mean(y);
            double stdev = get_stdev(y, mean);
            
            int n = x.Length;
            
            double[] normdist = new double[n];
            
            for (int i = 0; i < n; i++)
            {
                normdist[i] = 1 / (Math.Sqrt(2 * Math.PI) * stdev) * Math.Exp(-0.5 * Math.Pow((x[i] - mean) / stdev, 2));


            }

            return normdist;
        }

    }
}
