﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Data.OleDb;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class FormToolkit
    {
        bool curve_fit_exists = false;
        bool differencing_exists = false;
        bool detrend_spline_exists = false;

        int differencing_order = 1;

        #region curve fitting 
        #region detrend curve fitting page layout

        public void fill_combobox_with_datatablenames_detrend()
        {
            comboBox_stationarity_detrend_dataseriesname.Items.Clear();


            foreach (DataRow dr in datamaster.Tables["Sites"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["ID"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Interpolation"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Standardize"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["CumulativeDeparture"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Differencing"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["CurveFitting"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Residualsnameout"].ToString());
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["Curvenameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["SSA_vectors"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["RCnameoutVector"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["EOF_SPC_vectors"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_stationarity_detrend_dataseriesname.Items.Add(dr["SPCnameoutVector"].ToString());
                }
            }
          
        }


        private void comboBox_stationarity_detrend_dataseriesname_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.selecteddataseriesname_stationarity_detrend_input = (string)comboBox_stationarity_detrend_dataseriesname.SelectedItem;
            this.selecteddataseriesname_stationarity_detrend_trend_output = (string)comboBox_stationarity_detrend_dataseriesname.SelectedItem;
            this.selecteddataseriesname_stationarity_detrend_residuals_output = (string)comboBox_stationarity_detrend_dataseriesname.SelectedItem;

            this.selecteddataseriesname_stationarity_detrend_trend_output = selecteddataseriesname_stationarity_detrend_trend_output + "_curv_trd"; // add default suffix
            this.selecteddataseriesname_stationarity_detrend_residuals_output = selecteddataseriesname_stationarity_detrend_residuals_output + "_curv_res"; // add default suffix

            comboBox_detrend_curvefitting_method.SelectedItem = "Linear Fit";
            //this.selecteddataseriesname_detrend_spline_trend_output = selecteddataseriesname_stationarity_detrend_input + "_spl_trd"; // differencing trend suffix
            //this.selecteddataseriesname_detrend_spline_residuals_output = selecteddataseriesname_stationarity_detrend_input + "_spl_res"; // differencing trend suffix

            this.textBox_seriesname_trend_output.Text = this.selecteddataseriesname_stationarity_detrend_trend_output;
            this.textBox_seriesname_residuals_output.Text = this.selecteddataseriesname_stationarity_detrend_residuals_output;

            curve_fit_exists = change_viewdatatable(selecteddataseriesname_stationarity_detrend_trend_output);
            if (curve_fit_exists)
            {
                button_stationarity_plot_curve_trend_dataseries.Enabled = true;
                button_stationarity_plot_residuals.Enabled = true;
                button_view_datatable_residuals.Enabled = true;
            }
            else
            {
                button_stationarity_plot_curve_trend_dataseries.Enabled = false;
                button_stationarity_plot_residuals.Enabled = false;
                button_view_datatable_residuals.Enabled = false;
            }
        }

        private void comboBox_detrend_curvefitting_method_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBox = (ComboBox)sender;
            selecteddetrendmethod = (string)comboBox_detrend_curvefitting_method.SelectedItem;
        }

        #endregion detrend curve fitting page layout





        #region detrend curve fitting define inputs
        private void textBox_seriesname_trend_output_Validating(object sender, CancelEventArgs e)
        {
            selecteddataseriesname_stationarity_detrend_trend_output = textBox_seriesname_trend_output.Text;
        }
        #endregion detrend curve fitting define outputs



        #region detrend curve fitting actions

        private void button_calculate_trend_residuals_Click(object sender, EventArgs e)
        {
            try
            {
                double[] data_x;
                double[] data_y;

                int m;

                data_x = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_stationarity_detrend_input, 0);
                data_y = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_stationarity_detrend_input, 1);

                int num_points = data_x.Length;
                double[] trend_y = new double[num_points];

                double[] residuals = new double[num_points];

                update_progress_textbox("Fitting the curve...");

                if (selecteddetrendmethod == "Linear Fit")
                {
                    m = 2; //for linear fit
                    for (int i = 0; i < num_points; i++)
                    {
                        trend_y[i] = CPreprocessDataFactory.CDetrendFactory.get_trend(data_x, data_y, m, data_x[i]);
                    }
                    //selecteddataseriesname_preprocess_interpolated = selecteddataseriesname_preprocess_to_interpolate + "_natural_cubic_spline";
                }

                if (selecteddetrendmethod == "2nd Order Polynomial Fit")
                {
                    m = 3; //for 2nd order fit 
                    for (int i = 0; i < num_points; i++)
                    {
                        trend_y[i] = CPreprocessDataFactory.CDetrendFactory.get_trend(data_x, data_y, m, data_x[i]);
                    }
                    //selecteddataseriesname_preprocess_interpolated = selecteddataseriesname_preprocess_to_interpolate + "_natural_cubic_spline";
                }

                if (selecteddetrendmethod == "3rd Order Polynomial Fit")
                {
                    m = 4; //for 3nd order fit 
                    for (int i = 0; i < num_points; i++)
                    {
                        trend_y[i] = CPreprocessDataFactory.CDetrendFactory.get_trend(data_x, data_y, m, data_x[i]);
                    }
                    //selecteddataseriesname_preprocess_interpolated = selecteddataseriesname_preprocess_to_interpolate + "_natural_cubic_spline";
                }

                if (selecteddetrendmethod == "4th Order Polynomial Fit")
                {
                    m = 5; //for 4nd order fit 
                    for (int i = 0; i < num_points; i++)
                    {
                        trend_y[i] = CPreprocessDataFactory.CDetrendFactory.get_trend(data_x, data_y, m, data_x[i]);
                    }
                    //selecteddataseriesname_preprocess_interpolated = selecteddataseriesname_preprocess_to_interpolate + "_natural_cubic_spline";
                }

                // get residuals
                //data_x = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_stationarity_detrend_input, 0);
                //data_y = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_stationarity_detrend_input, 1);

                //trend_y = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_stationarity_detrend_trend_output, 1);


                update_progress_textbox("Getting the residuals...");

                for (int i = 0; i < num_points; i++)
                {
                    residuals[i] = data_y[i] - trend_y[i];
                }

                save_row_curvefitting_table(selecteddataseriesname, selecteddataseriesname_stationarity_detrend_input,
                    selecteddataseriesname_stationarity_detrend_trend_output, selecteddataseriesname_stationarity_detrend_residuals_output,
                    selecteddetrendmethod);

                get_new_datatable_in_datamaster(selecteddataseriesname_stationarity_detrend_trend_output);
                get_new_datatable_in_datamaster(selecteddataseriesname_stationarity_detrend_residuals_output);

                // put trend into a table in datamaster
                get_datatable_from_twoColArray(data_x, trend_y, selecteddataseriesname_stationarity_detrend_trend_output);
                // put residuals into a table in datamaster
                get_datatable_from_twoColArray(data_x, residuals, selecteddataseriesname_stationarity_detrend_residuals_output);

                selectedseries_mostrecent = selecteddataseriesname_stationarity_detrend_residuals_output;



                update_datagridview_progress();

                // add datatable to comboboxes
                fill_combobox_with_datatablenames_Describe_data();
                fill_combobox_with_datatablenames_interpolate_data();
                fill_combobox_with_datatablenames_standardize_data();
                fill_combobox_with_datatablenames_cumulative_departure();
                fill_combobox_with_datatablenames_ACF();
                fill_combobox_with_datatablenames_detrend();
                fill_combobox_with_datatablenames_differencing();
                fill_combobox_with_datatablenames_spectralanalysis();
                fill_combobox_with_datatablenames_MEM();
                fill_combobox_with_datatablenames_SSA();
                fill_combobox_with_datatablenames_AR();
                fill_combobox_with_datatablenames_regression_predictor();
                fill_combobox_with_datatablenames_regression_predicand();

                if (automaticallyGeneratePlotsToolStripMenuItem.Checked == true)
                {
                    // Open the form for plotting data. Pass the datamaster dataset to it. 
                    Form_series_plot form_plot = new Form_series_plot(datamaster, selecteddataseriesname_stationarity_detrend_residuals_output, this);
                    form_plot.Show();
                }

                curve_fit_exists = change_viewdatatable(selecteddataseriesname_stationarity_detrend_residuals_output);

                if (curve_fit_exists)
                {
                    button_stationarity_plot_curve_trend_dataseries.Enabled = true;
                    button_stationarity_plot_residuals.Enabled = true;
                    button_view_datatable_residuals.Enabled = true;
                    update_progress_textbox("Curve fitting complete.");
                }
                else
                {
                    button_stationarity_plot_curve_trend_dataseries.Enabled = false;
                    button_stationarity_plot_residuals.Enabled = false;
                    button_view_datatable_residuals.Enabled = false;
                    update_progress_textbox("Something went wrong. The curve fitting is not complete.");
                }
            }
            catch
            {
                update_progress_textbox("Something went wrong. The curve fitting is not complete.");
            }
        }

        private void button_stationarity_plot_trend_dataseries_Click(object sender, EventArgs e)
        {
            // Open the form for plotting data. Pass the datamaster dataset to it. 
            Form_multi_plot form_multi_plot = new Form_multi_plot(datamaster, selecteddataseriesname_stationarity_detrend_input, selecteddataseriesname_stationarity_detrend_trend_output, this);
            form_multi_plot.Show();
        }

        private void button_stationarity_plot_residuals_Click(object sender, EventArgs e)
        {
            // Open the form for plotting data. Pass the datamaster dataset to it. 
            Form_series_plot form_plot = new Form_series_plot(datamaster, selecteddataseriesname_stationarity_detrend_residuals_output, this);
            form_plot.Show();
        }

        private void button_view_datatable_residuals_Click(object sender, EventArgs e)
        {
            // Open the form for plotting data. Pass the datamaster dataset to it. 
            Form_datagrid form_grid = new Form_datagrid(datamaster, selecteddataseriesname_stationarity_detrend_residuals_output);
            form_grid.Show();
        }
        #endregion detrend curve fitting actions
        #endregion curve fitting

        #region detrend differencing
        
        
        #region differencing define inputs

        public void fill_combobox_with_datatablenames_differencing()
        {
            comboBox_detrend_differencing_dataseriesname.Items.Clear();


            foreach (DataRow dr in datamaster.Tables["Sites"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["ID"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Interpolation"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Standardize"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["CumulativeDeparture"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["Differencing"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Seriesnameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["CurveFitting"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Residualsnameout"].ToString());
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["Curvenameout"].ToString());
                }
            }
            foreach (DataRow dr in datamaster.Tables["SSA_vectors"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["RCnameoutVector"].ToString());
                }
            }

            foreach (DataRow dr in datamaster.Tables["EOF_SPC_vectors"].Rows)
            {
                if (dr["ID"].ToString() == selecteddataseriesname)
                {
                    comboBox_detrend_differencing_dataseriesname.Items.Add(dr["SPCnameoutVector"].ToString());
                }
            }
            //if (selectedseries_mostrecent == null)
            //{
            //    comboBox_detrend_differencing_dataseriesname.SelectedItem = selecteddataseriesname;
            //}
            //else
            //{
            //    comboBox_detrend_differencing_dataseriesname.SelectedItem = selectedseries_mostrecent;
            //}

            //foreach (DataTable table in datamaster.Tables)
            //{
            //    if (table.TableName != "sites" & table.TableName != "preprocess" & table.TableName != "stationarity"
            //        & table.TableName != "Fourier" & table.TableName != "SSA" & table.TableName != "EOF"
            //        & table.TableName != "Regression" & table.TableName != "AR" & table.TableName != "Projection")
            //    {
            //        if (table.TableName.Length >= selecteddataseriesname.Length)
            //        {
            //            if (table.TableName.Substring(0, selecteddataseriesname.Length) == this.selecteddataseriesname)
            //            {
            //                comboBox_detrend_differencing_dataseriesname.Items.Add(table.TableName);
            //                comboBox_detrend_differencing_dataseriesname.Items.IndexOf(table.TableName);
            //            }
            //        }
            //    }
            //}
            //comboBox_stationarity_detrend_dataseriesname.SelectedItem = selecteddataseriesname;

            //comboBox_detrend_differencing_order.SelectedItem = "1";
        }

        private void comboBox_detrend_differencing_dataseriesname_SelectedIndexChanged(object sender, EventArgs e)
        {

            this.selecteddataseriesname_detrend_differencing_input = (string)comboBox_detrend_differencing_dataseriesname.SelectedItem;
            this.selecteddataseriesname_detrend_differencing_trend_output = (string)comboBox_detrend_differencing_dataseriesname.SelectedItem;
            this.selecteddataseriesname_detrend_differencing_residuals_output = (string)comboBox_detrend_differencing_dataseriesname.SelectedItem;

            this.selecteddataseriesname_detrend_differencing_trend_output = selecteddataseriesname_detrend_differencing_trend_output + "_diff_trd"; // differencing trend suffix
            this.selecteddataseriesname_detrend_differencing_residuals_output = selecteddataseriesname_detrend_differencing_residuals_output + "_diff_res"; // differencing trend suffix

            differencing_exists = change_viewdatatable(selecteddataseriesname_detrend_differencing_trend_output);
            
            if (differencing_exists)
            {
                //button_detrend_differencing_plot_trend_dataseries.Enabled = true;
                button_detrend_differencing_plot_residuals.Enabled = true;
                button_detrend_differencing_show_datatable.Enabled = true;
            }
            else
            {
                //button_detrend_differencing_plot_trend_dataseries.Enabled = false;
                button_detrend_differencing_plot_residuals.Enabled = false;
                button_detrend_differencing_show_datatable.Enabled = false;
            }
            
            this.textBox_detrend_differencing_residuals_output.Text = this.selecteddataseriesname_detrend_differencing_residuals_output;
            this.textBox_detrend_differencing_trend_output.Text = this.selecteddataseriesname_detrend_differencing_trend_output;
        }

        private void comboBox_detrend_differencing_order_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                differencing_order = Convert.ToInt32(comboBox_detrend_differencing_order.SelectedItem);
            }
            catch
            {
                // must be a integer
            }
        }        
        
        
        #endregion differencing define inputs

  

        #region differencing define outputs

        #endregion differencing define outputs

        #region differencing actions

        private void button_calculate_detrend_differencing_Click(object sender, EventArgs e)
        {
            double[] data_x;
            double[] data_y;

            double[] residuals;
            double[] xnew;

            data_x = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_detrend_differencing_input, 0);
            data_y = get_oneColArray_from_datatable(datamaster, selecteddataseriesname_detrend_differencing_input, 1);

            CPreprocessDataFactory.CDetrendFactory.get_differencing(data_x, data_y, differencing_order, out residuals, out xnew);

            save_row_differencing_table(selecteddataseriesname, selecteddataseriesname_detrend_differencing_input,
                selecteddataseriesname_detrend_differencing_residuals_output, differencing_order);

            //get_new_datatable_in_datamaster(selecteddataseriesname_detrend_differencing_trend_output);
            get_new_datatable_in_datamaster(selecteddataseriesname_detrend_differencing_residuals_output);

            // put trend into a table in datamaster
            //get_datatable_from_twoColArray(data_x, trend_y, selecteddataseriesname_stationarity_detrend_trend_output);
            // put residuals into a table in datamaster
            get_datatable_from_twoColArray(xnew, residuals, selecteddataseriesname_detrend_differencing_residuals_output);

            // add datatable to comboboxes

            selectedseries_mostrecent = selecteddataseriesname_detrend_differencing_residuals_output;


            update_datagridview_progress();

            fill_combobox_with_datatablenames_Describe_data();
            fill_combobox_with_datatablenames_interpolate_data();
            fill_combobox_with_datatablenames_standardize_data();
            fill_combobox_with_datatablenames_cumulative_departure();
            fill_combobox_with_datatablenames_ACF();
            fill_combobox_with_datatablenames_detrend();
            fill_combobox_with_datatablenames_differencing();
            fill_combobox_with_datatablenames_spectralanalysis();
            fill_combobox_with_datatablenames_MEM();
            fill_combobox_with_datatablenames_SSA();
            fill_combobox_with_datatablenames_AR();
            fill_combobox_with_datatablenames_regression_predictor();
            fill_combobox_with_datatablenames_regression_predicand();

            if (automaticallyGeneratePlotsToolStripMenuItem.Checked == true)
            {
                // Open the form for plotting data. Pass the datamaster dataset to it. 
                Form_series_plot form_plot = new Form_series_plot(datamaster, selecteddataseriesname_detrend_differencing_residuals_output, this);
                form_plot.Show();
            }

            differencing_exists = change_viewdatatable(selecteddataseriesname_detrend_differencing_residuals_output);

            if (differencing_exists)
            {
                //button_detrend_differencing_plot_trend_dataseries.Enabled = true;
                button_detrend_differencing_plot_residuals.Enabled = true;
                button_detrend_differencing_show_datatable.Enabled = true;
                update_progress_textbox("Differencing complete");
            }
            else
            {
                //button_detrend_differencing_plot_trend_dataseries.Enabled = false;
                button_detrend_differencing_plot_residuals.Enabled = false;
                button_detrend_differencing_show_datatable.Enabled = false;
                update_progress_textbox("Something went wrong. The differencing is not complete");
            }
        }

        private void button_detrend_differencing_plot_residuals_Click(object sender, EventArgs e)
        {
            // Open the form for plotting data. Pass the datamaster dataset to it. 
            Form_series_plot form_plot = new Form_series_plot(datamaster, selecteddataseriesname_detrend_differencing_residuals_output, this);
            form_plot.Show();
        }

        private void button_detrend_differencing_show_datatable_Click(object sender, EventArgs e)
        {
            // Open the form for plotting data. Pass the datamaster dataset to it. 
            Form_datagrid form_grid = new Form_datagrid(datamaster, selecteddataseriesname_detrend_differencing_residuals_output);
            form_grid.Show();
        }
        #endregion differencing actions



        #endregion detrend differencing

    }

}
