﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_eigen_spectrum_plot_EOF : Form
    {
        private Cdatamaster datamaster;
        private string selecteddataseries;
        private string selectederror;

        private FormToolkit formtoolkit;

        private double[] x; // array for holding x axis values
        private double[] y; // array for holding x axis values

        private double xmin;
        private double xmax;

        double[] error;
        double[] error_upper;
        double[] error_lower;

        Series series = new Series();
        Series errorBarSeries = new Series("ErrorBar");

        public Form_eigen_spectrum_plot_EOF(Cdatamaster datamasterToUse, string selecteddataseriesToUse, int colx, int coly, string selectederrorToUse, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;
            this.selectederror = selectederrorToUse;





            this.Text = this.selecteddataseries;
            this.formtoolkit = formtoolkitToUse;

            try
            {
                // create series
                this.series = chart_eigen_spectrum.Series.Add(this.selecteddataseries);
                //this.series = chart_eigen_spectrum.Series.Add("ErrorBar");

                // assign data to series

                string col0name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[colx].ColumnName);
                string col1name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[coly].ColumnName);

                this.x = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, colx);
                this.y = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, coly);

                this.error = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selectederror, 1);

                //this.xmin = this.x.Min();
                this.xmin = 0;
                this.xmax = this.x.Max();

                // assign upper and lower errors
                this.error_upper = new double[this.x.Length];
                this.error_lower = new double[this.x.Length];

                errorBarSeries.ChartType = SeriesChartType.ErrorBar;
                // errorBarSeries.MarkerBorderColor = Color.Black;
                errorBarSeries.MarkerSize = 6;
                errorBarSeries.YValuesPerPoint = 3;
                //errorBarSeries.BorderColor = Color.FromArgb(180, 26, 59, 105);
                errorBarSeries.Color = Color.Black;
                //errorBarSeries.ShadowOffset = 1;
                errorBarSeries.MarkerStyle = MarkerStyle.None;
                errorBarSeries["PointWidth"] = ".7";


                for (int i = 0; i < x.Length; i++)
                {
                    this.error_upper[i] = this.y[i] + this.error[i];
                    this.error_lower[i] = this.y[i] - this.error[i];
                    errorBarSeries.Points.AddXY(this.x[i], this.error[i], this.error_lower[i], this.error_upper[i]);
                }

                chart_eigen_spectrum.Series.Add(errorBarSeries);

                // set up default plotting options 

                // Enable range selection and zooming end user interface
                chart_eigen_spectrum.ChartAreas[0].CursorX.IsUserEnabled = true;
                chart_eigen_spectrum.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                chart_eigen_spectrum.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                chart_eigen_spectrum.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
                chart_eigen_spectrum.ChartAreas[0].BackColor = Color.Transparent;

                chart_eigen_spectrum.ChartAreas[0].AxisX.Title = Convert.ToString(col0name);
                chart_eigen_spectrum.ChartAreas[0].AxisY.Title = Convert.ToString(col1name);

                chart_eigen_spectrum.ChartAreas[0].AxisX.RoundAxisValues();

                chart_eigen_spectrum.ChartAreas[0].AxisX.Minimum = this.xmin;
                chart_eigen_spectrum.ChartAreas[0].AxisX.Maximum = this.xmax;

                chart_eigen_spectrum.ChartAreas[0].AxisY.Minimum = 0;

                chart_eigen_spectrum.Legends[0].Docking = Docking.Top;
                chart_eigen_spectrum.Legends[0].Alignment = StringAlignment.Center;

                UpdateChartSettings();

            }
            catch
            {
                MessageBox.Show("This dataseries is empty. Please generate the values first");
            }




        }

        private void UpdateChartSettings()
        {
            chart_eigen_spectrum.Series[this.selecteddataseries].Points.DataBindXY(this.x, this.y);
            chart_eigen_spectrum.Series[this.selecteddataseries].ChartType = SeriesChartType.Point;
            chart_eigen_spectrum.Series[this.selecteddataseries]["PixelPointWidth"] = "1";
            chart_eigen_spectrum.Series[this.selecteddataseries].Color = Color.Black;
            chart_eigen_spectrum.Series[this.selecteddataseries].MarkerStyle = MarkerStyle.Circle;
            chart_eigen_spectrum.Series[this.selecteddataseries].MarkerSize = 5;
            chart_eigen_spectrum.Series[this.selecteddataseries].MarkerColor = Color.White;
            chart_eigen_spectrum.Series[this.selecteddataseries].MarkerBorderColor = Color.Black;
            chart_eigen_spectrum.Series[this.selecteddataseries].MarkerBorderWidth = 1;




            //chart_eigen_spectrum.Series["errorBarSeries"].Color = Color.Black;
            //chart_eigen_spectrum.Series["errorBarSeries"].MarkerSize = 10;
            //chart_eigen_spectrum.Series["errorBarSeries"].YValuesPerPoint = 10;
            //chart_eigen_spectrum.Series["errorBarSeries"].MarkerStyle = MarkerStyle.None;





            chart_eigen_spectrum.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
            chart_eigen_spectrum.ChartAreas[0].AxisY.MajorGrid.Enabled = false;
        }

        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void Form_plot_Load(object sender, EventArgs e)
        {

        }

        private void comboBox_chart_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            //string plottype;
            //plottype = (string)comboBox_chart_type.SelectedItem;
            ////chart1.Series[0].ChartType = SeriesChartType.Spline;            
            ////MessageBox.Show(plottype.ToString());

            //change_chart_type(plottype);
        }

        private void change_chart_type(string plottype)
        {

            // Set series chart type
            //if (plottype == "Point")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Point;
            //    MessageBox.Show(plottype);
            ////}

            //if (plottype == "Line")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Line;
            //}

            //if (plottype == "Spline")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Spline;
            //}
        }

        private void checkBox_showpoints_CheckedChanged(object sender, EventArgs e)
        {
            //    if (checkBox_showpoints.Checked == true)
            //    {
            //        chart1.Series[0].Enabled = true;
            //        chart1.Series[0].ChartType = SeriesChartType.Point;
            //    }
            //    if (checkBox_showpoints.Checked == false)
            //    {
            //        chart1.Series[0].Enabled = false;
            //    }
        }

        //private void checkBox_showpoints_CheckedChanged_1(object sender, EventArgs e)
        //{

        //}

        //private void checkBox_showline_CheckedChanged(object sender, EventArgs e)
        //{

        //}

        private void ToolStripMenuItem_close_Click(object sender, EventArgs e)
        {
            //Application.Exit();
            this.Close();
        }

        private void ToolStripMenuItem_save_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart_eigen_spectrum.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void ToolStripMenuItem_copy_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart_eigen_spectrum.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp);
        }

        private void ToolStripMenuItem_print_Click(object sender, EventArgs e)
        {
            chart_eigen_spectrum.Printing.Print(true);
        }

        private void ToolStripMenuItem_print_preview_Click(object sender, EventArgs e)
        {
            chart_eigen_spectrum.Printing.PrintPreview();
        }

        private void viewToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void viewPointsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //viewPointsToolStripMenuItem.Checked = !viewPointsToolStripMenuItem.Checked;
            //if (viewPointsToolStripMenuItem.Checked == true)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "ChartArea1";
            //}
            //if (viewPointsToolStripMenuItem.Checked == false)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "";
            //}
        }

        private void viewLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //viewLineToolStripMenuItem.Checked = !viewLineToolStripMenuItem.Checked;
            //if (viewLineToolStripMenuItem.Checked == true)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "ChartArea1";

            //}
            //if (viewLineToolStripMenuItem.Checked == false)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "";
            //}
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart_eigen_spectrum.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void cToolStripMenuItem_Click(object sender, EventArgs e)
        {
                        // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart_eigen_spectrum.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp);
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_eigen_spectrum.Printing.PrintPreview();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_eigen_spectrum.Printing.Print(true);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }



    }
}
