﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Climate_toolkit
{
    public partial class Form_export_raster : Form
    {
     private Cdatamaster datamaster;
        private FormToolkit formtoolkit;
        private string dataseriesname;
        private string selectedseries;

        private double xll = 0;
        private double yll = 0;
        private double cellsize = 0;
        private double nodata = 0;

        private int firstloop = 0;
        private int secondloop = 0;
        
        public Form_export_raster(Cdatamaster datamasterToUse, string dataseriesnameToUse, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.dataseriesname = dataseriesnameToUse;
            this.formtoolkit = formtoolkitToUse;

            // fill in values for the comboboxes
            fill_combobox_with_datatablenames_export_raster();
            comboBox_export_raster_dataseriesname.Text = dataseriesname;
            comboBox_export_raster_dataseriesname.SelectedText = dataseriesname.ToString();
            fill_combobox_with_datatablenames_export_raster_seriesnames();
            comboBox_export_raster_select_series.SelectedText = dataseriesname.ToString();

            if (comboBox_export_raster_dataseriesname.Text == "")
            {
                comboBox_export_raster_select_series.Enabled = false;
            }
            else
            {
                comboBox_export_raster_select_series.Enabled = true;
            }
           

            // fill the select series combobox with series associated with the dataseriesname


        }

        public void fill_combobox_with_datatablenames_export_raster()
        {
            comboBox_export_raster_dataseriesname.Items.Clear();
            foreach (DataRow dr in datamaster.Tables["sites"].Rows)
            {
                {
                    comboBox_export_raster_dataseriesname.Items.Add(dr["SiteID"].ToString());
                }
            }
        }

        public void fill_combobox_with_datatablenames_export_raster_seriesnames()
        {
            comboBox_export_raster_select_series.Items.Clear();
            foreach (DataTable table in datamaster.Tables)
            {
                if (table.TableName != "sites" & table.TableName != "preprocess" & table.TableName != "stationarity"
                    & table.TableName != "Fourier" & table.TableName != "SSA" & table.TableName != "EOF"
                    & table.TableName != "Regression" & table.TableName != "AR" & table.TableName != "Projection"
                    & table.TableName != "ProjectionSum")
                {
                    if (table.TableName.Length >= dataseriesname.Length)
                    {
                        if (table.TableName.Substring(0, dataseriesname.Length) == this.dataseriesname)
                        {
                            comboBox_export_raster_select_series.Items.Add(table.TableName);
                            comboBox_export_raster_select_series.Items.IndexOf(table.TableName);
                        }
                    }
                }
            }
        }



        private void comboBox_export_raster_dataseriesname_SelectedIndexChanged(object sender, EventArgs e)
        {
            fill_combobox_with_datatablenames_export_raster_seriesnames();
        }

        private void button_export_raster_select_save_file_Click(object sender, EventArgs e)
        {
            // get the size of the array

            //foreach (DataRow dr in datamaster.Tables["sites"].Rows)
            //{
            //    if (dr["ID"].ToString() == dataseriesname && dr["DataForm"].ToString() == "matrix")
            //    {
            //        firstloop = Convert.ToInt32(dr["Dim1"]);
            //        secondloop = Convert.ToInt32(dr["Dim2"]);
            //    }
            //    break;
            //}

            get_selected_output_dimensions(out firstloop, out secondloop);

            double[,] X = new double[firstloop, secondloop];// holds the array;            

            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.DefaultExt = "asc";
            saveFileDialog1.Filter = "Save File (*.asc)| *.asc";
            saveFileDialog1.Title = "Save File";
            saveFileDialog1.ShowDialog();

            // If the file name is not an empty string open it for saving.
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                double[] A = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selectedseries, 0);
                int count = 0;
                for (int j = 0; j < secondloop; j++)
                {
                    for (int i = 0; i < firstloop; i++)
                    {
                        X[i,j] = A[count];
                        count++;
                    }
                }

                this.formtoolkit.write_ascii_raster(X, saveFileDialog1.FileName, xll, yll, cellsize, nodata);
            }
        }

        private void button_export_raster_close_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void comboBox_export_raster_select_series_SelectedIndexChanged(object sender, EventArgs e)
        {
            selectedseries = comboBox_export_raster_select_series.Text.ToString();
        }

        private void textBox_export_raster_xllcorner_Validating(object sender, CancelEventArgs e)
        {
            xll = Convert.ToDouble(textBox_export_raster_xllcorner.Text);
        }

        private void textBox_export_raster_yllcorner_Validating(object sender, CancelEventArgs e)
        {
            yll = Convert.ToDouble(textBox_export_raster_yllcorner.Text);
        }

        private void textBox_export_raster_cellsize_Validating(object sender, CancelEventArgs e)
        {
            cellsize = Convert.ToDouble(textBox_export_raster_cellsize.Text);
        }

        private void textBox_export_raster_NODATAvalue_Validating(object sender, CancelEventArgs e)
        {
            nodata = Convert.ToDouble(textBox_export_raster_nodata.Text);
        }


        private void get_selected_output_dimensions(out int nrow, out int ncol)
        {
            // get the size of the selected output from the 


            nrow = 0;
            ncol = 0;


            //foreach (DataTable table in datamaster.Tables)
            //{
            //    if (table.TableName == this.selectedseries)
            //    {

            //        nrow = table.Rows.Count;
            //        ncol = table.Columns.Count;
            //    }
            //}



            foreach (DataRow dr in datamaster.Tables["sites"].Rows)
            {
                if (dr["ID"].ToString() == this.dataseriesname && dr["DataForm"].ToString() == "matrix")
                {
                    nrow = Convert.ToInt32(dr["Dim1"]);
                    ncol = Convert.ToInt32(dr["Dim2"]);
                }
                break;
            }


        }

        //private void comboBox_export_csv_grid_select_series_SelectedIndexChanged(object sender, EventArgs e)
        //{

        //}
        //private void button_export_csv_grid_close_Click(object sender, EventArgs e)
        //{

        //}

        //private void button_export_csv_grid_select_save_file_Click(object sender, EventArgs e)
        //{

        //}


        //private void comboBox_export_csv_grid_dataseriesname_SelectedIndexChanged(object sender, EventArgs e)
        //{

        //}
    }
}
