﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Collections;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Net;
using System.Text.RegularExpressions;

namespace Climate_toolkit
{
    public partial class Form_import_NWIS : Form
    {
        private Cdatamaster datamaster;
        private FormToolkit formtoolkit;
        private string Rawtablename;
        private Dictionary<string, CSWSite> SWSites;

        private string northlimit;
        private string southlimit;
        private string westlimit;
        private string eastlimit;
        string sitename;
        string sitenumber;

        string station_nm;
        string sitetype;
        string altitude;
        string lat_dec;
        string long_dec;
        string lat_dms;
        string long_dms;
        string latlong_dms_datum;
        string latlong_dec_datum;
        string begindate;
        string enddate;
        string countnum;

        string gwcount = "30";

        string selectedsiteid;

        string selectedsitename;
        string selectedsitetype;
        string selectedsitelat;
        string selectedsitelong;
        string selectedsitedeclat;
        string selectedsitedeclong;
        string selectedsitelatlongdatum;
        string selectedsitedeclatlongdatum;
        string selectedsitealt;
        string selectedsitegwbegindate;
        string selectedsitegwenddate;
        string selectedsitecount;

        ArrayList sitename_list = new ArrayList();
        ArrayList sitetype_list = new ArrayList();
        ArrayList sitelat_list = new ArrayList();
        ArrayList sitelong_list = new ArrayList();
        ArrayList sitedeclat_list = new ArrayList();
        ArrayList sitedeclong_list = new ArrayList();
        ArrayList sitelatlongdatum_list = new ArrayList();
        ArrayList sitedeclatlongdatum_list = new ArrayList();
        ArrayList sitealt_list = new ArrayList();
        ArrayList sitegwbegindate_list = new ArrayList();
        ArrayList sitegwenddate_list = new ArrayList();
        ArrayList sitecount_list = new ArrayList();

        // constructor
        public Form_import_NWIS(Cdatamaster datamasterToUse, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            datamaster = datamasterToUse; // make a local copy this subform holds on to
            //dr = drToUse;
            formtoolkit = formtoolkitToUse;
        }

        public Form_import_NWIS()
        {
            InitializeComponent();
        }

        private void getWLData(string siteID)
        {
            // This part defines the data table
            DataTable dtTable = new DataTable();
            dtTable.Columns.Add("Date", typeof(DateTime));
            dtTable.Columns.Add("Value", typeof(double));

            // build the web request and query nwis
            string requestStr = "http://nwis.waterdata.usgs.gov/nwis/gwlevels?site_no=" + siteID + "&agency_cd=USGS&format=rdb";
            WebClient webClient = new WebClient();
            Stream instr = webClient.OpenRead(requestStr);
            StreamReader stRead = new StreamReader(instr);

            // Start processing the data
            // This part skips over the comments
            string inln = stRead.ReadLine();
            while (inln != null && inln.StartsWith("#"))
            {
                inln = stRead.ReadLine();
            }

            // This part reads the field names and gets the index to the different fields
            string[] fldNames = inln.Split('\t');
            int valIndex = Array.IndexOf(fldNames, "lev_va");
            int dateIndex = Array.IndexOf(fldNames, "lev_dt");

            // This skips over the field discription line
            stRead.ReadLine();

            // This starts reading the data
            inln = stRead.ReadLine();
            while (inln != null)
            {
                double wlVal;

                string[] vals = inln.Split('\t');
                if (!string.IsNullOrEmpty(vals[valIndex]) && double.TryParse(vals[valIndex], out wlVal))
                {
                    DateTime dtVal;
                    DateTime.TryParse(vals[dateIndex], out dtVal);
                    inln = inln.Replace('\t', ',');
                    dtTable.Rows.Add(dtVal, wlVal);
                }

                inln = stRead.ReadLine();
            }
            stRead.Close();

            dtTable.TableName = siteID;
            // add the datatable to datamaster
            if (datamaster.Tables.Contains(dtTable.TableName) == false)
            {

                datamaster.Tables.Add(dtTable);
                Rawtablename = dtTable.TableName;

                //fill a row in the sites table with a description of the data
                //formtoolkit.fill_data_descriptions_from_NWIS(siteID, station_nm, lat_dms, long_dms, latlong_dms_datum, lat_dec, long_dec, latlong_dec_datum, sitetype, altitude);
                formtoolkit.fill_data_descriptions_from_NWIS(siteID, selectedsitename, selectedsitelat, selectedsitelong, selectedsitelatlongdatum, selectedsitedeclat, selectedsitedeclong, selectedsitedeclatlongdatum, selectedsitetype, selectedsitealt);

                // add the ID and name of datatable to "sites" table in datamaster               
                //formtoolkit.fill_row_sites_table(siteID, Rawtablename, "vector", dtTable.Rows.Count, 0, 0);
                // add the data to a raw data table
                formtoolkit.get_rawdatatable = Rawtablename;

                formtoolkit.get_ID = siteID;
                formtoolkit.get_selecteddataseriesname = siteID;
                formtoolkit.get_rawdatatable = Rawtablename;
                formtoolkit.setcombobox_dataseriesname(siteID);

                formtoolkit.update_progress_textbox("Data loaded from NWIS...");
            }
            else
            {
                MessageBox.Show("This data series has already been read");
            }


            formtoolkit.fill_combobox_with_datatablenames_selecteddataseries();
            // set the selecteddataseries combobox to the imported series name
            formtoolkit.setcombobox_dataseriesname(siteID);

            formtoolkit.fill_combobox_with_datatablenames_selecteddataseries();
            formtoolkit.fill_combobox_with_datatablenames_predictand_selecteddataseries();
            formtoolkit.fill_combobox_with_datatablenames_Describe_data();
            formtoolkit.fill_combobox_with_datatablenames_preprocess_data();
            formtoolkit.fill_combobox_with_datatablenames_interpolate_data();
            formtoolkit.fill_combobox_with_datatablenames_standardize_data();
            formtoolkit.fill_combobox_with_datatablenames_cumulative_departure();
            formtoolkit.fill_combobox_with_datatablenames_spectralanalysis();
            formtoolkit.fill_combobox_with_datatablenames_MEM();
            formtoolkit.fill_combobox_with_datatablenames_SSA();
            formtoolkit.fill_combobox_with_datatablenames_ACF();
            formtoolkit.fill_combobox_with_datatablenames_detrend();
            formtoolkit.fill_combobox_with_datatablenames_differencing();
            formtoolkit.fill_combobox_with_datatablenames_SSA();
            formtoolkit.fill_combobox_with_datatablenames_AR();
            formtoolkit.fill_combobox_with_datatablenames_regression_predictor();

            // set the selecteddataseries combobox to the imported series name
            formtoolkit.set_comboboxes_to_new_series();

            formtoolkit.get_ID = siteID;
            formtoolkit.get_rawdatatable = Rawtablename;
            Close();
            //return dtTable;
        }
        private void getSWData(string siteID)
        {
            // This part defines the data table
            DataTable dtTable = new DataTable();
            dtTable.Columns.Add("Date", typeof(DateTime));
            dtTable.Columns.Add("Value", typeof(double));

            XNamespace ns = "http://www.cuahsi.org/waterML/1.1/";

            XElement root = XElement.Load(SWSites[siteID].queryURL);
            var retvals = from elem in root.Elements(ns + "timeSeries")
                         where elem.Element(ns + "variable").Element(ns + "variableCode").Value == "00060"
                         select elem;

            foreach (var elem in retvals)
            {
                XElement vals = elem.Element(ns + "values");
                IEnumerable<XElement> qretvals =
                    from qval in vals.Elements(ns + "value")
                    select qval;
                foreach (XElement valElem in qretvals)
                {
                    DateTime dt;
                    if (DateTime.TryParse(valElem.Attribute("dateTime").Value,out dt))
                    {
                        dtTable.Rows.Add(dt, valElem.Value);
                    }
                }
                break;
            }
            dtTable.TableName = siteID;
            if (datamaster.Tables.Contains(dtTable.TableName) == false)
            {

                datamaster.Tables.Add(dtTable);
                Rawtablename = dtTable.TableName;
                formtoolkit.fill_data_descriptions_from_NWIS(siteID, SWSites[siteID].siteName, "", "", "", SWSites[siteID].lat_dec, SWSites[siteID].long_dec, SWSites[siteID].latlong_dec_datum, SWSites[siteID].dataType, "");
                formtoolkit.setcombobox_dataseriesname(siteID);

                formtoolkit.fill_combobox_with_datatablenames_selecteddataseries();
                formtoolkit.fill_combobox_with_datatablenames_predictand_selecteddataseries();
                formtoolkit.fill_combobox_with_datatablenames_Describe_data();
                formtoolkit.fill_combobox_with_datatablenames_preprocess_data();
                formtoolkit.fill_combobox_with_datatablenames_interpolate_data();
                formtoolkit.fill_combobox_with_datatablenames_standardize_data();
                formtoolkit.fill_combobox_with_datatablenames_cumulative_departure();
                formtoolkit.fill_combobox_with_datatablenames_spectralanalysis();
                formtoolkit.fill_combobox_with_datatablenames_MEM();
                formtoolkit.fill_combobox_with_datatablenames_SSA();
                formtoolkit.fill_combobox_with_datatablenames_ACF();
                formtoolkit.fill_combobox_with_datatablenames_detrend();
                formtoolkit.fill_combobox_with_datatablenames_differencing();
                formtoolkit.fill_combobox_with_datatablenames_SSA();
                formtoolkit.fill_combobox_with_datatablenames_AR();
                formtoolkit.fill_combobox_with_datatablenames_regression_predictor();

                // set the selecteddataseries combobox to the imported series name
                formtoolkit.set_comboboxes_to_new_series();
                formtoolkit.get_ID = siteID;
                formtoolkit.get_rawdatatable = Rawtablename;
                Close();
            }
        }
        
        private string coordinateformat
        {
            get
            {
                if (radioButton_LatLong_DMS.Checked) return "dms";
                if (radioButton_LatLong_DD.Checked) return "decimal_degrees";
                return null;
            }
        }
        private string sitenumber_match
        {
            get
            {
                if (radioButton_sitenumber_exactmatch.Checked) return "exact";
                if (radioButton_sitenumber_matchanypart.Checked) return "anywhere";
                if (radioButton_sitenumber_matchstart.Checked) return "beginning";
                return null;
            }
        }

        private string sitename_match
        {
            get
            {
                if (radioButton_sitename_matchanypart.Checked) return "anywhere";
                if (radioButton_sitename_matchstart.Checked) return "beginning";
                return null;
            }
        }

        private string getnwisdate(DateTime dt)
        {
            return dt.Year.ToString("0000") + "-" + dt.Month.ToString("00") + "-" + dt.Day.ToString("00");
        }
        //http://nwis.waterdata.usgs.gov/nwis/gwlevels?state_cd=az&gw_count_nu=300&format=station_list&sort_key=site_no&group_key=NONE&date_format=YYYY-MM-DD&rdb_compression=file&list_of_search_criteria=state_cd%2Cobs_count_nu
        private string getWebSite()
        {
            string weblink = "";
            if (radioButton_DataCat_GW.Checked)
            #region Make GW URL.
            {
                //weblink = "http://nwis.waterdata.usgs.gov/nwis/gwlevels?search_station_nm=d-23-21%2006&search_station_nm_match_type=beginning&format=sitefile_output&sort_key=site_no&group_key=NONE&sitefile_output_format=xml&column_name=agency_cd&column_name=site_no&column_name=station_nm&column_name=lat_va&column_name=long_va&column_name=dec_lat_va&column_name=dec_long_va&column_name=gw_begin_date&column_name=gw_end_date&column_name=gw_count_nu&begin_date=&end_date=&date_format=YYYY-MM-DD&rdb_compression=file&list_of_search_criteria=lat_long_bounding_box";
                weblink = "http://nwis.waterdata.usgs.gov/nwis/gwlevels?";

                if (!string.IsNullOrEmpty(sitenumber))
                {
                    weblink = weblink + "search_site_no=" + sitenumber + "&" + "search_site_no_match_type=" + sitenumber_match + "&";
                }
                else if (!string.IsNullOrEmpty(sitename))
                {
                    weblink = weblink + "search_station_nm=" + sitename + "&" + "search_station_nm_match_type=" + sitename_match + "&";
                }

                if (!string.IsNullOrEmpty(westlimit) && !string.IsNullOrEmpty(northlimit) && !string.IsNullOrEmpty(eastlimit) && !string.IsNullOrEmpty(southlimit))
                {
                    weblink = weblink
                    + "nw_longitude_va=" + westlimit + "&"
                    + "nw_latitude_va=" + northlimit + "&"
                    + "se_longitude_va=" + eastlimit + "&"
                    + "se_latitude_va=" + southlimit + "&"
                    + "coordinate_format=" + coordinateformat + "&";
                }

                if (!string.IsNullOrEmpty(gwcount))
                {
                    weblink = weblink + "gw_count_nu=" + gwcount + "&";
                }
                if (checkBoxUseDateRange.Checked)
                {
                    weblink = weblink
                    + "begin_date=" + getnwisdate(dateTimePickerStartDate.Value) + "&"
                    + "end_date=" + getnwisdate(dateTimePickerEndDate.Value) + "&"
                    + "date_format=YYYY-MM-DD&";
                }

                weblink = weblink
                         + "format=sitefile_output&"
                         + "sort_key=site_no&"
                         + "group_key=NONE&"
                         + "sitefile_output_format=xml&"
                         + "column_name=agency_cd&"
                         + "column_name=site_no&"
                         + "column_name=station_nm&"
                         + "column_name=site_tp_cd&"
                         + "column_name=lat_va&"
                         + "column_name=long_va&"
                         + "column_name=dec_lat_va&"
                         + "column_name=dec_long_va&"
                         + "column_name=lat_long_datum_cd&"
                         + "column_name=dec_lat_long_datum_cd&"
                         + "column_name=alt_va&"
                         + "column_name=gw_begin_date&"
                         + "column_name=gw_end_date&"
                         + "column_name=gw_count_nu&"

                         + "rdb_compression=file&";
            }
            #endregion
            else
            #region Make SW URL.
            {
                //http://waterservices.usgs.gov/nwis/site/?format=rdb&bBox=-83.000000,36.500000,-81.000000,38.500000&seriesCatalogOutput=true&outputDataTypeCd=dv&siteType=ST
                //http://waterservices.usgs.gov/nwis/site/?format=rdb&bBox=-83.000000,36.500000,-81.000000,38.500000&seriesCatalogOutput=true&outputDataTypeCd=dv&hasDataTypeCd=dv&parameterCd=00060&siteType=ST
                weblink = "http://waterservices.usgs.gov/nwis/site/?format=rdb&";

                if (!string.IsNullOrEmpty(westlimit) && !string.IsNullOrEmpty(northlimit) && !string.IsNullOrEmpty(eastlimit) && !string.IsNullOrEmpty(southlimit))
                {
                    weblink = weblink
                              + "bBox=" + westlimit + "," + southlimit + "," + eastlimit + "," + northlimit + "&";
                }

                if (checkBoxUseDateRange.Checked)
                {
                    weblink = weblink
                    + "begin_date=" + getnwisdate(dateTimePickerStartDate.Value) + "&"
                    + "end_date=" + getnwisdate(dateTimePickerEndDate.Value) + "&"
                    + "date_format=YYYY-MM-DD&";
                }

                weblink = weblink
                          + "siteType=ST&"
                          + "seriesCatalogOutput=true&"
                          + "outputDataTypeCd=dv&"
                          + "hasDataTypeCd=dv&"
                          + "parameterCd=00060";


            }
            #endregion

            return weblink;
        }
        private bool formIsValid()
        {
            bool isvalid = false;
            if (!string.IsNullOrEmpty(sitenumber) || !string.IsNullOrEmpty(sitename) ||
                (!string.IsNullOrEmpty(northlimit) && !string.IsNullOrEmpty(southlimit) &&
                !string.IsNullOrEmpty(westlimit) && !string.IsNullOrEmpty(eastlimit)))
            {
                isvalid = true;
            }
            return isvalid;
        }

        private void textBox_siteID_TextChanged(object sender, EventArgs e)
        {
            sitenumber = textBox_siteID.Text;
        }

        private void button_OK_Click(object sender, EventArgs e)
        {
            try
            {
                #region Query GW data
                if (radioButton_DataCat_GW.Checked)
                {
                    if (formIsValid())
                    {
                        dataGridView1.Rows.Clear();
                        string query = getWebSite();
                        WebClient webClient1 = new WebClient();
                        Stream myStream = webClient1.OpenRead(query);
                        XmlDocument xdoc = new XmlDocument();
                        XmlElement docElem;

                        //StreamReader sr = new StreamReader(myStream);
                        //string retstr = sr.ReadToEnd();
                        //MessageBox.Show(retstr);

                        xdoc.Load(myStream);
                        docElem = xdoc.DocumentElement;

                        foreach (XmlNode xnode in docElem.ChildNodes)
                        {
                            if (xnode.Name == "error")
                            {
                                MessageBox.Show(xnode.InnerText);
                            }
                            else
                            {

                                int indx = dataGridView1.Rows.Add();
                                DataGridViewRow dgvrow = dataGridView1.Rows[indx];

                                foreach (XmlNode chXnode in xnode.ChildNodes)
                                {
                                    switch (chXnode.Name)
                                    {
                                        case "site_no":
                                            dgvrow.Cells["Column_sitenumber"].Value = chXnode.InnerText;
                                            break;
                                        case "station_nm":
                                            dgvrow.Cells["Column_sitename"].Value = chXnode.InnerText;
                                            station_nm = chXnode.InnerText;
                                            sitename_list.Add(station_nm);
                                            break;
                                        case "site_tp_cd":
                                            sitetype = chXnode.InnerText;
                                            sitetype_list.Add(sitetype);
                                            break;
                                        case "lat_va":
                                            lat_dms = chXnode.InnerText;
                                            sitelat_list.Add(lat_dms);
                                            break;
                                        case "long_va":
                                            long_dms = chXnode.InnerText;
                                            sitelong_list.Add(long_dms);
                                            break;
                                        case "dec_lat_va":
                                            lat_dec = chXnode.InnerText;
                                            sitedeclat_list.Add(lat_dec);
                                            break;
                                        case "dec_long_va":
                                            long_dec = chXnode.InnerText;
                                            sitedeclong_list.Add(long_dec);
                                            break;
                                        case "lat_long_datum_cd":
                                            latlong_dms_datum = chXnode.InnerText;
                                            sitelatlongdatum_list.Add(latlong_dms_datum);
                                            break;
                                        case "dec_lat_long_datum_cd":
                                            latlong_dec_datum = chXnode.InnerText;
                                            sitedeclatlongdatum_list.Add(latlong_dec_datum);
                                            break;
                                        case "alt_va":
                                            altitude = chXnode.InnerText;
                                            sitealt_list.Add(altitude);
                                            break;
                                        case "gw_begin_date":
                                            dgvrow.Cells["Column_begindate"].Value = chXnode.InnerText;
                                            begindate = chXnode.InnerText;
                                            sitegwbegindate_list.Add(begindate);
                                            break;
                                        case "gw_end_date":
                                            dgvrow.Cells["Column_enddate"].Value = chXnode.InnerText;
                                            enddate = chXnode.InnerText;
                                            sitegwenddate_list.Add(enddate);
                                            break;
                                        case "gw_count_nu":
                                            dgvrow.Cells["Column_numberobs"].Value = chXnode.InnerText;
                                            countnum = chXnode.InnerText;
                                            sitecount_list.Add(countnum);
                                            break;

                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        MessageBox.Show(
                            "Please enter either a site number, site name, or lat-long box to search for.");
                    }
                }
                #endregion
                #region Query SW data
                else
                {
                    if (formIsValid())
                    {
                        dataGridView1.Rows.Clear();
                        if (SWSites == null)
                        {
                            SWSites = new Dictionary<string, CSWSite>();
                        }
                        string query = getWebSite();
                        WebClient webClient1 = new WebClient();
                        Stream myStream = webClient1.OpenRead(query);
                        using (StreamReader sr = new StreamReader(myStream))
                        {
                            string inln = sr.ReadLine();
                            //skip over the comment lines
                            while (sr.Peek() >= 0 && inln.StartsWith("#"))
                            {
                                inln = sr.ReadLine();
                            }

                            CSWIndexes indexes = new CSWIndexes(inln);

                            while (sr.Peek() >= 0)
                            {
                                inln = sr.ReadLine();
                                if (inln.Contains("00060"))
                                {
                                    CSWSite site = new CSWSite(indexes, inln);
                                    SWSites[site.siteNumber] = site;
                                    int indx = dataGridView1.Rows.Add();
                                    DataGridViewRow dgvrow = dataGridView1.Rows[indx];

                                    dgvrow.Cells["Column_sitenumber"].Value = site.siteNumber;
                                    dgvrow.Cells["Column_sitename"].Value = site.siteName;
                                    dgvrow.Cells["Column_begindate"].Value = site.startDt;
                                    dgvrow.Cells["Column_enddate"].Value = site.endDt;
                                    dgvrow.Cells["Column_numberobs"].Value = site.swCnt;

                                }
                            }
                        }

                    }
                }
                #endregion
            }
            catch (System.Net.WebException weberror)
            {
                MessageBox.Show(weberror.Message, "Connection error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message + "\n" + exception.StackTrace);
            }


        }

        private string mkDMS(string instr, bool checklong)
        {
            string retstr = null;
            instr = instr.Trim();
            string chkstr = instr.Replace(" ", "");
            //Regex reNum = new Regex(@"^\d+$");
            Regex reNum = new Regex(@"[-+]?\b\d+\b");
            if (reNum.Match(chkstr).Success)
            {
                if (instr.StartsWith("-"))
                {
                    instr = instr.Substring(1);
                }

                if (instr.Contains(" "))
                {
                    string[] invals = instr.Split();
                    if (invals.Length > 3)
                    {
                        string[] tmpvals = invals;
                        invals = new string[3];
                        Array.Copy(tmpvals, 0, invals, 0, 3);
                    }
                    if (invals.Length == 3)
                    {
                        retstr = string.Join(" ", invals);
                    }
                    else
                    {
                        retstr = string.Join(" ", invals) + " 0";
                    }
                }
                else if (instr.Length == 6)
                {
                    retstr = instr.Substring(0, 2) + " " + instr.Substring(2, 2) + " " +
                             instr.Substring(4);
                }
                else if (instr.Length == 7)
                {
                    retstr = instr.Substring(0, 3) + " " + instr.Substring(3, 2) + " " +
                             instr.Substring(5);
                }

                if (instr.Contains(" "))
                {
                    string[] invals = retstr.Split();
                    if (invals.Length == 3)
                    {
                        int chkd;
                        int chkm;
                        double chks;

                        if (int.TryParse(invals[0], out chkd) && int.TryParse(invals[1], out chkm) && double.TryParse(invals[2], out chks))
                        {
                            if (checklong)
                            {
                                if (chkd > 180 || chkm >= 60 || chks >= 60)
                                {
                                    retstr = null;
                                }
                            }
                            else
                            {
                                if (chkd > 90 || chkm >= 60 || chks >= 60)
                                {
                                    retstr = null;
                                }
                            }
                        }
                    }
                }

            }

            return retstr;
        }

        private void textBox_northlimit_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_northlimit.Text))
            {
                e.Cancel = true;
                if (coordinateformat == "dms")
                {
                    string dms = mkDMS(textBox_northlimit.Text, true);
                    if (!string.IsNullOrEmpty(dms))
                    {
                        textBox_northlimit.Text = dms;
                        e.Cancel = false;
                        northlimit = dms;
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter values in DD MM SS format.");
                        textBox_northlimit.Text = "";
                    }
                }
                else
                {
                    double val;
                    if (double.TryParse(textBox_northlimit.Text, out val))
                    {
                        double absval = Math.Abs(val);
                        if (absval > 0 && absval < 90)
                        {
                            e.Cancel = false;
                            northlimit = val.ToString();
                        }
                        else
                        {
                            MessageBox.Show("Invalid values. Please enter valid values in D.DDDD format.");
                            textBox_northlimit.Text = "";
                        }
                    }
                }
            }
            else
            {
                northlimit = null;
            }
        }

        private void textBox_westlimit_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_westlimit.Text))
            {
                e.Cancel = true;
                if (coordinateformat == "dms")
                {
                    string dms = mkDMS(textBox_westlimit.Text, false);
                    if (!string.IsNullOrEmpty(dms))
                    {
                        textBox_westlimit.Text = dms;
                        e.Cancel = false;
                        westlimit = dms;
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter values in DD MM SS format.");
                        textBox_westlimit.Text = "";
                    }
                }
                else
                {
                    double val;
                    if (double.TryParse(textBox_westlimit.Text, out val))
                    {
                        double absval = Math.Abs(val);
                        if (absval > 0 && absval < 180)
                        {
                            e.Cancel = false;
                            westlimit = val.ToString();
                        }
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter valid values in D.DDDD format.");
                        textBox_westlimit.Text = "";
                    }
                }
            }
            else
            {
                westlimit = null;
            }

        }

        private void textBox_eastlimit_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_eastlimit.Text))
            {
                e.Cancel = true;
                if (coordinateformat == "dms")
                {
                    string dms = mkDMS(textBox_eastlimit.Text, false);
                    if (!string.IsNullOrEmpty(dms))
                    {
                        textBox_eastlimit.Text = dms;
                        e.Cancel = false;
                        eastlimit = dms;
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter values in DD MM SS format.");
                        textBox_eastlimit.Text = "";
                    }
                }
                else
                {
                    double val;
                    if (double.TryParse(textBox_eastlimit.Text, out val))
                    {
                        double absval = Math.Abs(val);
                        if (absval > 0 && absval < 180)
                        {
                            e.Cancel = false;
                            eastlimit = val.ToString();
                        }
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter valid values in D.DDDD format.");
                        textBox_eastlimit.Text = "";
                    }
                }
            }
            else
            {
                eastlimit = null;
            }

        }

        private void textBox_southlimit_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_southlimit.Text))
            {
                e.Cancel = true;
                if (coordinateformat == "dms")
                {
                    string dms = mkDMS(textBox_southlimit.Text, true);
                    if (!string.IsNullOrEmpty(dms))
                    {
                        textBox_southlimit.Text = dms;
                        e.Cancel = false;
                        southlimit = dms;
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter values in DD MM SS format.");
                        textBox_southlimit.Text = "";
                    }
                }
                else
                {
                    double val;
                    if (double.TryParse(textBox_southlimit.Text, out val))
                    {
                        double absval = Math.Abs(val);
                        if (absval > 0 && absval < 90)
                        {
                            e.Cancel = false;
                            southlimit = val.ToString();
                        }
                    }
                    else
                    {
                        MessageBox.Show("Invalid values. Please enter valid values in D.DDDD format.");
                        textBox_southlimit.Text = "";
                    }
                }
            }
            else
            {
                southlimit = null;
            }

        }

        private void button_getdata_Click(object sender, EventArgs e)
        {
            selectedsiteid = null;
            selectedsitename = null;
            selectedsitetype = null;
            selectedsitelat = null;
            selectedsitelong = null;
            selectedsitedeclat = null;
            selectedsitedeclong = null;
            selectedsitelatlongdatum = null;
            selectedsitedeclatlongdatum = null;
            selectedsitealt = null;
            selectedsitegwbegindate = null;
            selectedsitegwenddate = null;
            selectedsitecount = null;

            int count = 0;
            foreach (DataGridViewRow dgrow in dataGridView1.Rows)
            {
                if (dgrow.Cells[0].Value != null && (bool)dgrow.Cells[0].Value == true)
                {
                    selectedsiteid = dgrow.Cells[1].Value.ToString();
                    selectedsitename = sitename_list[count].ToString();
                    selectedsitetype = sitetype_list[count].ToString();
                    selectedsitelat = sitelat_list[count].ToString();
                    selectedsitelong = sitelong_list[count].ToString();
                    selectedsitedeclat = sitedeclat_list[count].ToString();
                    selectedsitedeclong = sitedeclong_list[count].ToString();
                    selectedsitelatlongdatum = sitelatlongdatum_list[count].ToString();
                    selectedsitedeclatlongdatum = sitedeclatlongdatum_list[count].ToString();
                    selectedsitealt = sitealt_list[count].ToString();
                    selectedsitegwbegindate = sitegwbegindate_list[count].ToString();
                    selectedsitegwenddate = sitegwenddate_list[count].ToString();
                    selectedsitecount = sitecount_list[count].ToString();
                    break;
                }
                count++;

            }
            if (string.IsNullOrEmpty(selectedsiteid))
            {
                MessageBox.Show("Please select a site");
            }
            else
            {
                if (radioButton_DataCat_GW.Checked)
                {
                    getWLData(selectedsiteid);
                }
                else
                {
                    getSWData(selectedsiteid);
                }
            }
        }

        private void button_cancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void textBox_numberobs_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_numberobs.Text))
            {
                int cnt;
                if (int.TryParse(textBox_numberobs.Text, out cnt))
                {
                    gwcount = cnt.ToString();
                }
                else
                {
                    textBox_numberobs.Text = "0";
                    gwcount = "0";
                }
            }
            else
            {
                gwcount = null;
            }
        }

        private static string ConvertWhitespaceToSpacesRegex(string value)
        {
            //value = Regex.Replace(value, "[\n\r\t]", " ");
            value = Regex.Replace(value, " ", "%20");
            return value;
        }

        private void textBox_sitename_Validating(object sender, CancelEventArgs e)
        {
            sitename = ConvertWhitespaceToSpacesRegex(textBox_sitename.Text);
        }

        private void textBox_siteID_Validating(object sender, CancelEventArgs e)
        {
            if (!string.IsNullOrEmpty(textBox_siteID.Text))
            {
                Regex reNum = new Regex(@"^\d+$");
                if (!reNum.Match(textBox_siteID.Text).Success)
                {
                    MessageBox.Show("The site number can only be numbers. Most site numbers are 8 or 15-digits long.");
                    e.Cancel = true;
                }
            }
        }

        private void button_resetform_Click(object sender, EventArgs e)
        {
            radioButton_DataCat_GW.Checked = true;
            textBox_northlimit.Text = "";
            textBox_eastlimit.Text = "";
            textBox_southlimit.Text = "";
            textBox_westlimit.Text = "";
            radioButton_LatLong_DD.Checked = true;
            textBox_siteID.Text = "";
            radioButton_sitename_matchstart.Checked = true;
            textBox_sitename.Text = "";
            radioButton_sitenumber_matchstart.Checked = true;
            textBox_numberobs.Text = "30";
            northlimit = null;
            southlimit = null;
            westlimit = null;
            eastlimit = null;
            sitename = null;
            sitenumber = null;
            dataGridView1.Rows.Clear();
        }

        private void radioButton_DataCat_CheckedChanged(object sender, EventArgs e)
        {
            radioButton_LatLong_DMS.Enabled = radioButton_DataCat_GW.Checked;
            textBox_numberobs.Enabled = radioButton_DataCat_GW.Checked;
            label_numberobs.Enabled = radioButton_DataCat_GW.Checked;
            groupBox_siteidentifier.Enabled = radioButton_DataCat_GW.Checked;
            if (!radioButton_LatLong_DMS.Enabled && radioButton_LatLong_DMS.Checked)
            {
                radioButton_LatLong_DD.Checked = true;
            }
        }

    }

}
