using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Odbc;
using System.Data.SqlClient;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Climate_toolkit
{
	public partial class Form_import_csv : Form
	{
        private Cdatamaster datamaster;
        private FormToolkit formtoolkit;
        private string Rawtablename;

        string col0name = "time";
        string col1name = "value"; //default values of column names

        int numberOfRows;

        //private DataRow dr;

        // constructor
        public Form_import_csv(Cdatamaster datamasterToUse, FormToolkit formtoolkitToUse)
		{
			InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            //this.dr = drToUse;
            this.formtoolkit = formtoolkitToUse;
		}
        
		private string fileCSV;		//full file name
		private string dirCSV;		//directory of file to import
		private string fileNevCSV;	//name (with extension) of file to import - field

		public string FileNevCSV	//name (with extension) of file to import - property
		{
			get { return fileNevCSV; }
			set { fileNevCSV = value; }
		}

		private string strFormat;	//CSV separator character
		private string strEncoding; //Encoding of CSV file

		private long rowCount = 0;	//row number of source file
		


		// Browses file with OpenFileDialog control

		private void btnFileOpen_Click(object sender, EventArgs e)
		{
			OpenFileDialog openFileDialogCSV = new OpenFileDialog();

			openFileDialogCSV.InitialDirectory = Application.ExecutablePath.ToString();
            openFileDialogCSV.Filter = "TXT files (*.txt)|*.txt| CSV files (*.csv)|*.csv";
            //openFileDialogCSV.Filter = "CSV files (*.csv)|*.csv|TXT files (*.txt)|*.txt|  All files (*.*)|*.*";
			openFileDialogCSV.FilterIndex = 1;
			openFileDialogCSV.RestoreDirectory = true;

			if (openFileDialogCSV.ShowDialog() == DialogResult.OK)
			{
				this.txtFileToImport.Text = openFileDialogCSV.FileName.ToString();
			}

            this.textBox_col0name.Enabled = true;
            this.textBox_col1name.Enabled = true;
            this.textBox_numrows.Text = "100";

            //loadPreview();

		}



		// Delimiter character selection
		private void Format()
		{
			try
			{

                if (rdbComma.Checked)
                {
                    strFormat = "Delimited(,)";
                }
                else if (rdbSpace.Checked)
                {
                    strFormat = "Delimited( )";
                }
                else if (rdbSemicolon.Checked)
                {
                    strFormat = "Delimited(;)";
                }
                else if (rdbTab.Checked)
                {
                    strFormat = "TabDelimited";
                }
                else if (rdbSeparatorOther.Checked)
                {
                    strFormat = "Delimited(" + txtSeparatorOtherChar.Text.Trim() + ")";
                }
                else
                {
                    strFormat = "Delimited(,)";
                }


			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Format");
			}
			finally
			{
			}
		}


		// Encoding selection
		private void Encoding()
		{
			try
			{

				if (rdbAnsi.Checked)
				{
					strEncoding = "ANSI";
				}
				else if (rdbUnicode.Checked)
				{
					strEncoding = "Unicode";
				}
				else if (rdbOEM.Checked)
				{
					strEncoding = "OEM";
				}
				else
				{
					strEncoding = "ANSI";
				}


			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Encoding");
			}
			finally
			{
			}
		}



		/* Schema.ini File (Text File Driver)

		When the Text driver is used, the format of the text file is determined by using a
		schema information file. The schema information file, which is always named Schema.ini
		and always kept in the same directory as the text data source, provides the IISAM 
		with information about the general format of the file, the column name and data type
		information, and a number of other data characteristics*/

		private void writeSchema()
		{
			try
			{
				FileStream fsOutput = new FileStream(this.dirCSV + "\\schema.ini", FileMode.Create, FileAccess.Write);
				StreamWriter srOutput = new StreamWriter(fsOutput);
				string s1, s2, s3, s4, s5;

				s1 = "[" + this.FileNevCSV + "]";
				s2 = "ColNameHeader=" + chkFirstRowColumnNames.Checked.ToString();
				s3 = "Format=" + this.strFormat;
				s4 = "MaxScanRows=25";
				s5 = "CharacterSet=" + this.strEncoding;

				srOutput.WriteLine(s1.ToString() + "\r\n" + s2.ToString() + "\r\n" + s3.ToString() + "\r\n" + s4.ToString() + "\r\n" + s5.ToString());
				srOutput.Close();
				fsOutput.Close();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "writeSchema");
			}
			finally
			{ }
		}

		/*
		 * Loads the csv file into a DataSet.
		 * 
		 * If the numberOfRows parameter is -1, it loads all rows, otherwise it
		 * loads the first specified number of rows (for preview)
		 */

		public DataTable LoadCSV(int numberOfRows)
		{
			DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            DataTable dt_temp = new DataTable();
            int numrows = 0;

			try
			{
				// Creates and opens an ODBC connection
                OdbcConnectionStringBuilder builder = new OdbcConnectionStringBuilder();
                builder["Driver"] = "Microsoft Text Driver (*.txt; *.csv)";
                builder["dbq"] = this.dirCSV.Trim();
                builder["Extension"] = "asc,csv,tab,txt";



                string strConnString = builder.ConnectionString;

                //string strConnString = "Driver={Microsoft Text Driver (*.txt; *.csv; *.dat)};Dbq=" + this.dirCSV.Trim() + ";Extensions=asc,csv,tab,txt,dat;Persist Security Info=False";
                string sql_select;
                string sql_select_temp;

				OdbcConnection conn;
				conn = new OdbcConnection(strConnString.Trim());
				conn.Open();

				//Creates the select command text
				if (chkFirstRowColumnNames.Checked == true)
                {
                    if (numberOfRows == -1)
				    {
					    sql_select = "select * from [" + this.FileNevCSV.Trim() + "]";
				    }
				    else
				    {
					    sql_select = "select top " + numberOfRows + " * from [" + this.FileNevCSV.Trim() + "]";
				    }
                }
                else
                {
                    if (numberOfRows == -1)
                    {
                        sql_select = "select * from [" + this.FileNevCSV.Trim() + "]";
                    }
                    else
                    {
                        sql_select = "select top " + numberOfRows + " * from [" + this.FileNevCSV.Trim() + "]";
                    }
                }

                // find out how many rows are in the dataseries
                sql_select_temp = "select * from [" + this.FileNevCSV.Trim() + "]";
                OdbcDataAdapter obj_oledb_da_temp = new OdbcDataAdapter(sql_select_temp, conn);
                obj_oledb_da_temp.Fill(dt_temp);
                numrows = dt_temp.Rows.Count;
                textBox_dataserieslength.Text = numrows.ToString();


				//Creates the data adapter
				OdbcDataAdapter obj_oledb_da = new OdbcDataAdapter(sql_select, conn);

                obj_oledb_da.Fill(dt);

                //Fills dataset with the records from CSV file
                //obj_oledb_da.Fill(ds, "csv");
                
				//closes the connection
				conn.Close();

                //If the first row does not have column names, then rename them based on user input
                if (chkFirstRowColumnNames.Checked == false)
                {
                    dt.Columns[0].ColumnName = Convert.ToString(col0name);
                    dt.Columns[1].ColumnName = Convert.ToString(col1name);
                }

			}
			catch (Exception e) //Error
			{
				MessageBox.Show(e.Message, "Error - LoadCSV",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
			return dt;
		}


		// Checks if a file was given.
		private bool fileCheck()
		{
			if ((fileCSV == "") || (fileCSV == null) || (dirCSV == "") || (dirCSV == null) || (FileNevCSV == "") || (FileNevCSV == null))
			{
				MessageBox.Show("Select a file to load first!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}
			else
			{
				return true;
			}
		}




		private void btnPreview_Click(object sender, EventArgs e)
		{
			loadPreview();
		}


        /* Loads the preview of CSV file in the DataGridView control.
         */

		private void loadPreview()
		{
			try
			{
				// select format, encoding, an write the schema file
				Format();
				Encoding();
				writeSchema();

				// loads the first 100 rows from CSV file, and fills the
				// DataGridView control.
				this.dataGridView_preView.DataSource = LoadCSV(numberOfRows);
                //this.dataGridView_preView.DataMember = "csv";
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "Error - loadPreview", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

 		/*
		 *  shows the progress of import operation
		 */

        //private void OnSqlRowsCopied(object sender, SqlRowsCopiedEventArgs e)
        //{
        //    this.lblProgress.Text = "Imported: " + e.RowsCopied.ToString() + "/" + this.rowCount.ToString() + " row(s)";
        //    this.lblProgress.Refresh();
        //}


		/*
		 * Makes file and directory information, and suggests a table name
		 */

		private void tbFile_TextChanged(object sender, EventArgs e)
		{
			// full file name
			this.fileCSV = this.txtFileToImport.Text;

			// creates a System.IO.FileInfo object to retrive information from selected file.
			System.IO.FileInfo fi = new System.IO.FileInfo(this.fileCSV);
			// retrives directory
			this.dirCSV = fi.DirectoryName.ToString();
			// retrives file name with extension
			this.FileNevCSV = fi.Name.ToString();

			// database table name
			this.txt_siteID.Text = fi.Name.Substring(0, fi.Name.Length - fi.Extension.Length).Replace(" ", "_");
		}

		private void txtSeparatorOtherChar_TextChanged(object sender, EventArgs e)
		{
			this.rdbSeparatorOther.Checked = true;
		}

        private void Form_import_csv_Load(object sender, EventArgs e)
        {

        }

        private void txtTableName_TextChanged(object sender, EventArgs e)
        {

        }

        private void button3_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnOKLoadCSV_Click_1(object sender, EventArgs e)
        {
            DataTable dt;
            //DataRow dr;
            try
            {
                dt = LoadCSV(-1);
                //dt.TableName = "dat_"+this.txt_siteID.Text;
                dt.TableName = this.txt_siteID.Text;
                // add the datatable to datamaster
                if (this.datamaster.Tables.Contains(dt.TableName) == false)
                {
                    this.datamaster.Tables.Add(dt);
                    this.Rawtablename = dt.TableName;

                    // add the ID and name of datatable to "sites" table in datamaster               
                    this.formtoolkit.fill_row_sites_table(txt_siteID.Text, Rawtablename, "vector", dt.Rows.Count, 0, 0);
                    // add the data to a raw data table
                    formtoolkit.get_rawdatatable = this.Rawtablename;

                    formtoolkit.get_ID = this.txt_siteID.Text;
                    formtoolkit.get_selecteddataseriesname = this.txt_siteID.Text;
                    formtoolkit.get_rawdatatable = this.Rawtablename;
                    this.formtoolkit.setcombobox_dataseriesname(txt_siteID.Text);

                    //add the siteID to the dataseries combobox

                    //this.formtoolkit.clear_combobox_selecteddataseries();
                    //this.formtoolkit.update_comboboxes();

                    this.formtoolkit.fill_combobox_with_datatablenames_selecteddataseries();
                    this.formtoolkit.fill_combobox_with_datatablenames_predictand_selecteddataseries();
                    this.formtoolkit.fill_combobox_with_datatablenames_Describe_data();
                    this.formtoolkit.fill_combobox_with_datatablenames_preprocess_data();
                    this.formtoolkit.fill_combobox_with_datatablenames_interpolate_data();
                    this.formtoolkit.fill_combobox_with_datatablenames_standardize_data();
                    this.formtoolkit.fill_combobox_with_datatablenames_cumulative_departure();
                    this.formtoolkit.fill_combobox_with_datatablenames_spectralanalysis();
                    this.formtoolkit.fill_combobox_with_datatablenames_MEM();
                    this.formtoolkit.fill_combobox_with_datatablenames_SSA();
                    this.formtoolkit.fill_combobox_with_datatablenames_ACF();
                    this.formtoolkit.fill_combobox_with_datatablenames_detrend();
                    this.formtoolkit.fill_combobox_with_datatablenames_differencing();
                    this.formtoolkit.fill_combobox_with_datatablenames_SSA();
                    this.formtoolkit.fill_combobox_with_datatablenames_AR();
                    this.formtoolkit.fill_combobox_with_datatablenames_regression_predictor();

                    //this.formtoolkit.fill_comboboxwith_datatablenames();

                    // set the selecteddataseries combobox to the imported series name
                    this.formtoolkit.set_comboboxes_to_new_series();
                    this.formtoolkit.update_defaults();

                    this.formtoolkit.update_progress_textbox("File imported...");
                }
            }
            catch
            {
                MessageBox.Show("File wasn't imported correctly.");
            }
            Close();
        }
        public string ID
        {
            get { return this.txt_siteID.Text; }
        }

        public string rawtablename
        {
            get { return this.Rawtablename; }
        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void chkFirstRowColumnNames_CheckedChanged(object sender, EventArgs e)
        {
            if (chkFirstRowColumnNames.Checked == true)
            {
                textBox_col0name.Enabled = false;
                textBox_col1name.Enabled = false;

                //loadPreview();
            }
            else
            {
                textBox_col0name.Enabled = true;
                textBox_col1name.Enabled = true;
                //loadPreview();
            }
        }

        private void textBox_col0name_TextChanged(object sender, EventArgs e)
        {
            try
            {
                col0name = Convert.ToString(textBox_col0name.Text);
            }
            catch
            {
            }
        }

        private void textBox_col1name_TextChanged(object sender, EventArgs e)
        {
            try
            {
                col1name = Convert.ToString(textBox_col1name.Text);
            }
            catch
            {
            }
        }

        private void textBox_numrows_TextChanged(object sender, EventArgs e)
        {
            try
            {
                numberOfRows = Convert.ToInt32(textBox_numrows.Text);
            }
            catch
            {
            }
        }
		
	}
}