﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_series_plot : Form
    {
        private Cdatamaster datamaster;
        private string selecteddataseries;

        private FormToolkit formtoolkit;



        private double[] x; // array for holding x axis values
        private double[] y; // array for holding x axis values

        private double xmin;
        private double xmax;

        private double ymin;
        private double ymax;

        int nrow = 0;
        int ncol = 0;
        int ntime = 0;

        int randrow = 0;
        int randcol = 0;

        string col0name = null;
        string col1name = null;

        bool showplot_status = true;
        bool ismatrix = false;

        bool plot_points = false;
        bool plot_line = false;
        bool plot_points_line = true;

        string add_series_seriesname;

        Series series = new Series();


 
        public Form_series_plot(Cdatamaster datamasterToUse, string selecteddataseriesToUse,  FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;

            this.Text = this.selecteddataseries;
            this.formtoolkit = formtoolkitToUse;
            showplot_status = true;

            // find out if the series is a vector or matrix. if it's a vector, just plot the series that is part of 
            // the argument. If the series is a matrix, then randomly select a series to plot.


            string dataseriesname = get_dataseriesname(selecteddataseries);
            foreach (DataRow dr in datamaster.Tables["sites"].Rows)
            {
                if(dataseriesname == dr["ID"].ToString())
                {
                    string temp = dr["DataForm"].ToString();
                    if (dr["DataForm"].ToString() == "matrix")
                    {
                        
                        nrow = Convert.ToInt32(dr["Dim1"]);
                        ncol = Convert.ToInt32(dr["Dim2"]);
                        ntime = Convert.ToInt32(dr["Dim3"]);
                        ismatrix = true;
                    }
                }
            }

            if (ismatrix)
            {
                randrow = CProjectionFactory.get_random_int(0, nrow-1);
                randcol = CProjectionFactory.get_random_int(0, ncol - 1);
                this.x = new double[ntime];
                this.y = new double[ntime];
            }

            try
            {
                // create series
                this.series = chart1.Series.Add(this.selecteddataseries);

                // assign data to series

                if (ismatrix)
                {
                    double[] xtemp = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 0);
                    double[, ,] a = get_3loop_array_data(xtemp, nrow, ncol, ntime);
                    double[] xrand = new double[ntime];

                    for (int i = 0; i < ntime; i++)
                    {
                        xrand[i] = a[randrow, randcol, i];
                        this.x[i] = i;
                        this.y[i] = xrand[i];
                    }

                    col0name = "time";
                    col1name = "value";

                }
                else
                {
                    this.x = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 0);
                    this.y = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 1);
                    col0name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[0].ColumnName);
                    col1name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[1].ColumnName);
                }

                this.xmin = this.x.Min();
                this.xmin = Math.Round(this.xmin);
                this.xmax = this.x.Max();
                this.xmax = Math.Round(this.xmax, 0, MidpointRounding.AwayFromZero);

                this.ymin = this.y.Min();
                this.ymax = this.y.Max();

                if (this.xmin == this.xmax || this.ymin == this.ymax)
                {
                    showplot_status = false;
                }

                chart1.ChartAreas[0].AxisX.Crossing = 0; 

                // set up default plotting options 

                // Enable range selection and zooming end user interface
                chart1.ChartAreas[0].CursorX.IsUserEnabled = true;
                chart1.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                chart1.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                chart1.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
                chart1.ChartAreas[0].BackColor = Color.Transparent;

                chart1.ChartAreas[0].AxisX.Title = Convert.ToString(col0name);
                chart1.ChartAreas[0].AxisY.Title = Convert.ToString(col1name);

                //chart1.ChartAreas[0].AxisX.RoundAxisValues();
                //chart1.ChartAreas[0].AxisY.RoundAxisValues();

                //chart1.ChartAreas[0].AxisX.MaximumAutoSize = Convert.ToSingle(this.xmax);
                //chart1.ChartAreas[0].RecalculateAxesScale();

                chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
                chart1.ChartAreas[0].AxisX.Maximum = this.xmax;

                chart1.ChartAreas[0].AxisY.Minimum = this.ymin;
                chart1.ChartAreas[0].AxisY.Maximum = this.ymax;

                chart1.ChartAreas[0].AxisX.IntervalAutoMode = IntervalAutoMode.VariableCount;
                chart1.ChartAreas[0].AxisY.IntervalAutoMode = IntervalAutoMode.VariableCount;


                chart1.Legends[0].Docking = Docking.Top;
                chart1.Legends[0].Alignment = StringAlignment.Center;
                chart1.Legends[0].Font = new Font("Microsoft Sans Serif", 12);

                chart1.ChartAreas[0].AxisX.LabelStyle.Format = "###.##";
                chart1.ChartAreas[0].AxisY.LabelStyle.Format = "###.##";

                chart1.ChartAreas[0].AxisX.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisX.TitleFont = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.TitleFont = new Font("Microsoft Sans Serif", 12);


                formtoolkit.get_plot_status = showplot_status;

                UpdateChartSettings();


                
            }
            catch
            {
                MessageBox.Show("This dataseries is empty. Please generate the values first");
            }


            // Set ToolTips for Data Point Series
            chart1.Series[0].ToolTip = "#VALX, #VALY";
        }

        public void UpdateChartSettings()
        {
            if (plot_points_line)
            {
                chart1.Series[this.selecteddataseries].Points.DataBindXY(this.x, this.y);
                chart1.Series[this.selecteddataseries].ChartType = SeriesChartType.Line;
                chart1.Series[this.selecteddataseries]["PixelPointWidth"] = "1";
                chart1.Series[this.selecteddataseries].Color = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerStyle = MarkerStyle.Circle;
                chart1.Series[this.selecteddataseries].MarkerSize = 5;
                chart1.Series[this.selecteddataseries].MarkerColor = Color.White;
                chart1.Series[this.selecteddataseries].MarkerBorderColor = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerBorderWidth = 1;

                chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
                chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = false;

                //chart1.ChartAreas[0].AxisX.RoundAxisValues();
                //chart1.ChartAreas[0].AxisY.RoundAxisValues();
            }

            if (plot_points)
            {
                chart1.Series[this.selecteddataseries].Points.DataBindXY(this.x, this.y);
                chart1.Series[this.selecteddataseries].ChartType = SeriesChartType.Point;
                chart1.Series[this.selecteddataseries]["PixelPointWidth"] = "1";
                chart1.Series[this.selecteddataseries].Color = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerStyle = MarkerStyle.Circle;
                chart1.Series[this.selecteddataseries].MarkerSize = 5;
                chart1.Series[this.selecteddataseries].MarkerColor = Color.White;
                chart1.Series[this.selecteddataseries].MarkerBorderColor = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerBorderWidth = 1;

                chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
                chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = false;

                //chart1.ChartAreas[0].AxisX.RoundAxisValues();
                //chart1.ChartAreas[0].AxisY.RoundAxisValues();
            }

            if (plot_line)
            {
                chart1.Series[this.selecteddataseries].Points.DataBindXY(this.x, this.y);
                chart1.Series[this.selecteddataseries].ChartType = SeriesChartType.Line;
                chart1.Series[this.selecteddataseries]["PixelPointWidth"] = "1";
                chart1.Series[this.selecteddataseries].Color = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerStyle = MarkerStyle.Circle;
                chart1.Series[this.selecteddataseries].MarkerSize = 0;
                chart1.Series[this.selecteddataseries].MarkerColor = Color.White;
                chart1.Series[this.selecteddataseries].MarkerBorderColor = Color.Black;
                chart1.Series[this.selecteddataseries].MarkerBorderWidth = 0;

                chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
                chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = false;

                //chart1.ChartAreas[0].AxisX.RoundAxisValues();
                //chart1.ChartAreas[0].AxisY.RoundAxisValues();



            }

            chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
            chart1.ChartAreas[0].AxisX.Maximum = this.xmax;

            chart1.ChartAreas[0].AxisY.Minimum = this.ymin;
            chart1.ChartAreas[0].AxisY.Maximum = this.ymax;
        }




        /// <summary>
        /// Tooltip Event
        /// </summary>
        private void chart1_GetToolTipText(object sender, System.Windows.Forms.DataVisualization.Charting.ToolTipEventArgs e)
        {

            // Check selected chart element and set tooltip text
            switch (e.HitTestResult.ChartElementType)
            {
                case ChartElementType.DataPoint:
                    e.Text = "Data Point " + e.HitTestResult.PointIndex.ToString();
                    break;
            }

        }

        public bool get_plotstatus
        {
            get { return this.showplot_status; }
        }


        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void Form_plot_Load(object sender, EventArgs e)
        {

        }

        private void comboBox_chart_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            //string plottype;
            //plottype = (string)comboBox_chart_type.SelectedItem;
            ////chart1.Series[0].ChartType = SeriesChartType.Spline;            
            ////MessageBox.Show(plottype.ToString());

            //change_chart_type(plottype);
        }

        private void change_chart_type(string plottype)
        {

        }

        private void checkBox_showpoints_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void ToolStripMenuItem_close_Click(object sender, EventArgs e)
        {
            //Application.Exit();
            this.Close();
        }

        private void ToolStripMenuItem_save_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart1.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void ToolStripMenuItem_copy_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart1.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp); 
        }

        private void ToolStripMenuItem_print_Click(object sender, EventArgs e)
        {
            chart1.Printing.Print(true);
        }

        private void ToolStripMenuItem_print_preview_Click(object sender, EventArgs e)
        {
            chart1.Printing.PrintPreview();
        }

        private void viewToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }



        private string get_dataseriesname(string seriesname)
        {
            string dataseriesname = null;
            // look for seriesname in each of the main tables in datamaster
            foreach (DataRow dr in datamaster.Tables["Interpolation"].Rows)
            {
                if (dr["Seriesnameout"].ToString() == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["Standardize"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["CumulativeDeparture"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["ACF"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["Differencing"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["CurveFitting"].Rows)
            {
                if (dr["Residualsnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }

            foreach (DataRow dr in datamaster.Tables["Fourier"].Rows)
            {
                if (dr["Spectrumnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["MEM"].Rows)
            {
                if (dr["Spectrumnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["SSA"].Rows)
            {
                if (dr["Spectrumnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["EOF"].Rows)
            {
                if (dr["Eigennameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["AR"].Rows)
            {
                if (dr["SeriesnameSimout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["Regression"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["Projection"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }
            foreach (DataRow dr in datamaster.Tables["ProjectionSum"].Rows)
            {
                if (dr["Seriesnameout"] == seriesname)
                {
                    dataseriesname = dr["ID"].ToString();
                }
            }

            return dataseriesname;
        }

        public static double[, ,] get_3loop_array_data(double[] a, int dim1, int dim2, int dim3)
        {
            double[, ,] x = new double[dim1, dim2, dim3];

            int count = 0;

            //StreamReader sr = File.OpenText(datatablename);

            for (int q = 0; q < dim3; q++)
            {
                for (int j = 0; j < dim2; j++)
                {
                    for (int i = 0; i < dim1; i++)
                    {
                        //string strdata = sr.ReadLine();
                        //x[i, j, q] = Convert.ToDouble(strdata);
                        x[i, j, q] = a[count];
                        count++;
                    }
                }
            }

            return x;
        }

        private void viewPointsAndLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (viewPointsAndLineToolStripMenuItem.Checked == false)
            {
                viewPointsAndLineToolStripMenuItem.Checked = true;
            }
            plot_points_line = true;

            viewPointsToolStripMenuItem.Checked = false;
            viewLineToolStripMenuItem.Checked = false;

            plot_points = false;
            plot_line = false;

            UpdateChartSettings();
        }

        private void viewPointsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (viewPointsToolStripMenuItem.Checked == false)
            {
                viewPointsToolStripMenuItem.Checked = true;
            }
            plot_points = true;

            viewPointsAndLineToolStripMenuItem.Checked = false;
            viewLineToolStripMenuItem.Checked = false;

            plot_points_line = false;
            plot_line = false;

            UpdateChartSettings();
        }

        private void viewLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (viewLineToolStripMenuItem.Checked == false)
            {
                viewLineToolStripMenuItem.Checked = true;
            }
            plot_line = true;

            viewPointsAndLineToolStripMenuItem.Checked = false;
            viewPointsToolStripMenuItem.Checked = false;

            plot_points_line = false;
            plot_points = false;

            UpdateChartSettings();
        }

        private void addSeriesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form_select_series form_select_series = new Form_select_series(datamaster, this.selecteddataseries, this, formtoolkit);
            form_select_series.Show();

            // 
        }

        public string get_addseries_string
        {
            set { this.add_series_seriesname = value; }
        }

        public double get_xmin
        {
            set { this.xmin = value; }
        }

        public double get_xmax
        {
            set { this.xmax = value; }
        }

        public double get_ymin
        {
            set { this.ymin = value; }
        }

        public double get_ymax
        {
            set { this.ymax = value; }
        }

        private void axisPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form_plot_axis_properties form_plot_axis_properties = new Form_plot_axis_properties(this.xmin, this.xmax, this.ymin, this.ymax, this);
            form_plot_axis_properties.Show();
        }
 
    }
}
