﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_RCs_plot : Form
    {
        private Cdatamaster datamaster;
        private string selecteddataseries;

        private FormToolkit formtoolkit;

        private double[] x; // array for holding x axis values
        private double[] y; // array for holding x axis values

        private double[] RC;

        private double xmin;
        private double xmax;

        private double ymin;
        private double ymax;

        string rclist_string;

        Series series = new Series();

        public Form_RCs_plot(Cdatamaster datamasterToUse, string selecteddataseriesToUse, double[] RCToUse, int[] rclist, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;

            this.Text = this.selecteddataseries;
            this.formtoolkit = formtoolkitToUse;

            // get string for the rc list
            this.rclist_string= null;
            int rclist_length = rclist.Length;
            for(int i = 0; i< rclist_length; i++)
            {
                if (i == 0)
                {
                    rclist_string = rclist[i].ToString();
                }
                else
                {
                    rclist_string = rclist_string + ", " + rclist[i].ToString();
                }
            }
            rclist_string = "RC " + rclist_string;

            try
            {
                // create series
                this.series = chart1.Series.Add(this.selecteddataseries);
                this.series = chart1.Series.Add(this.rclist_string);

                // assign data to series

                string col0name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[0].ColumnName);
                string col1name = Convert.ToString(datamaster.Tables[this.selecteddataseries].Columns[1].ColumnName);

                this.x = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 0);
                this.y = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 1);
                this.RC = RCToUse;

                this.xmin = this.x.Min();
                this.xmin = Math.Round(this.xmin);
                this.xmax = this.x.Max();
                this.xmax = Math.Round(this.xmax, 0, MidpointRounding.AwayFromZero);

                this.ymin = this.y.Min();
                this.ymax = this.y.Max();

                // set up default plotting options 

                // Enable range selection and zooming end user interface
                chart1.ChartAreas[0].CursorX.IsUserEnabled = true;
                chart1.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                chart1.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                chart1.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
                chart1.ChartAreas[0].BackColor = Color.Transparent;

                chart1.ChartAreas[0].AxisX.Title = Convert.ToString(col0name);
                chart1.ChartAreas[0].AxisY.Title = Convert.ToString(col1name);

                chart1.ChartAreas[0].AxisX.RoundAxisValues();

                chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
                chart1.ChartAreas[0].AxisX.Maximum = this.xmax;

                chart1.Legends[0].Docking = Docking.Top;
                chart1.Legends[0].Alignment = StringAlignment.Center;

                chart1.ChartAreas[0].AxisX.IntervalAutoMode = IntervalAutoMode.VariableCount;
                chart1.ChartAreas[0].AxisY.IntervalAutoMode = IntervalAutoMode.VariableCount;

                chart1.ChartAreas[0].AxisX.LabelStyle.Format = "###.##";
                chart1.ChartAreas[0].AxisY.LabelStyle.Format = "###.##";

                UpdateChartSettings();
                chart1.Series[0].ToolTip = "#VALX, #VALY";
                chart1.Series[1].ToolTip = "#VALX, #VALY";

                chart1.ChartAreas[0].AxisX.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisX.TitleFont = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.TitleFont = new Font("Microsoft Sans Serif", 12);

            }
            catch
            {
                MessageBox.Show("This dataseries is empty. Please generate the values first");
            }




        }

        public void UpdateChartSettings()
        {
            chart1.Series[this.selecteddataseries].Points.DataBindXY(this.x, this.y);
            chart1.Series[this.selecteddataseries].ChartType = SeriesChartType.Line;
            chart1.Series[this.selecteddataseries]["PixelPointWidth"] = "1";
            chart1.Series[this.selecteddataseries].Color = Color.Black;
            //chart1.Series[this.selecteddataseries].MarkerStyle = MarkerStyle.Circle;
            //chart1.Series[this.selecteddataseries].MarkerSize = 5;
            //chart1.Series[this.selecteddataseries].MarkerColor = Color.White;
            //chart1.Series[this.selecteddataseries].MarkerBorderColor = Color.Black;
            //chart1.Series[this.selecteddataseries].MarkerBorderWidth = 1;

            chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
            chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = false;

            chart1.Series[this.rclist_string].Points.DataBindXY(this.x, this.RC);
            chart1.Series[this.rclist_string].ChartType = SeriesChartType.Line;
            chart1.Series[this.rclist_string]["PixelPointWidth"] = "1";
            chart1.Series[this.rclist_string].Color = Color.Red;

            chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
            chart1.ChartAreas[0].AxisX.Maximum = this.xmax;

            chart1.ChartAreas[0].AxisY.Minimum = this.ymin;
            chart1.ChartAreas[0].AxisY.Maximum = this.ymax;
        }

        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void Form_plot_Load(object sender, EventArgs e)
        {

        }

        private void comboBox_chart_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            //string plottype;
            //plottype = (string)comboBox_chart_type.SelectedItem;
            ////chart1.Series[0].ChartType = SeriesChartType.Spline;            
            ////MessageBox.Show(plottype.ToString());

            //change_chart_type(plottype);
        }

        private void change_chart_type(string plottype)
        {

            // Set series chart type
            //if (plottype == "Point")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Point;
            //    MessageBox.Show(plottype);
            ////}

            //if (plottype == "Line")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Line;
            //}

            //if (plottype == "Spline")
            //{
            //    chart1.Series[0].ChartType = SeriesChartType.Spline;
            //}
        }

        private void checkBox_showpoints_CheckedChanged(object sender, EventArgs e)
        {
            //    if (checkBox_showpoints.Checked == true)
            //    {
            //        chart1.Series[0].Enabled = true;
            //        chart1.Series[0].ChartType = SeriesChartType.Point;
            //    }
            //    if (checkBox_showpoints.Checked == false)
            //    {
            //        chart1.Series[0].Enabled = false;
            //    }
        }

        //private void checkBox_showpoints_CheckedChanged_1(object sender, EventArgs e)
        //{

        //}

        //private void checkBox_showline_CheckedChanged(object sender, EventArgs e)
        //{

        //}

        private void ToolStripMenuItem_close_Click(object sender, EventArgs e)
        {
            //Application.Exit();
            this.Close();
        }

        private void ToolStripMenuItem_save_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart1.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void ToolStripMenuItem_copy_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart1.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp);
        }

        private void ToolStripMenuItem_print_Click(object sender, EventArgs e)
        {
            chart1.Printing.Print(true);
        }

        private void ToolStripMenuItem_print_preview_Click(object sender, EventArgs e)
        {
            chart1.Printing.PrintPreview();
        }

        private void viewToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void viewPointsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //viewPointsToolStripMenuItem.Checked = !viewPointsToolStripMenuItem.Checked;
            //if (viewPointsToolStripMenuItem.Checked == true)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "ChartArea1";
            //}
            //if (viewPointsToolStripMenuItem.Checked == false)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "";
            //}
        }

        private void viewLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //viewLineToolStripMenuItem.Checked = !viewLineToolStripMenuItem.Checked;
            //if (viewLineToolStripMenuItem.Checked == true)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "ChartArea1";

            //}
            //if (viewLineToolStripMenuItem.Checked == false)
            //{
            //    chart1.Series[this.selecteddataseries].ChartArea = "";
            //}
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart1.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Application.Exit();
            this.Close();
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart1.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp); 
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Printing.PrintPreview();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Printing.Print(true);
        }

        private void axisPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form_plot_axis_properties form_plot_axis_properties = new Form_plot_axis_properties(this.xmin, this.xmax, this.ymin, this.ymax, this);
            form_plot_axis_properties.Show();
        }

        public double get_xmin
        {
            set { this.xmin = value; }
        }

        public double get_xmax
        {
            set { this.xmax = value; }
        }

        public double get_ymin
        {
            set { this.ymin = value; }
        }

        public double get_ymax
        {
            set { this.ymax = value; }
        }

    }
}
