﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_plot_histogram : Form
    {
        private Cdatamaster datamaster;
        private string selecteddataseries;
        double [] histogram;
        private FormToolkit formtoolkit;

        double[] x;
        double[] y;
        double[] normdist_scaled;
        double[] gammadist_scaled;
        double[] binvalues;
        double start;
        double end;
        int nbin;
        double binwidth;

        double numpoints;

        bool plot_normal;
        bool plot_gamma;

        Series series = new Series();

        public Form_plot_histogram(Cdatamaster datamasterToUse, string selecteddataseriesToUse, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();
            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries = selecteddataseriesToUse;
            this.formtoolkit = formtoolkitToUse;

            try
            {
                this.Text = this.selecteddataseries + " Histogram";

                this.series = chart_histogram.Series.Add("histogram");
                this.series = chart_histogram.Series.Add("normal");
                this.series = chart_histogram.Series.Add("gamma");

                // assign parameters for the histogram
                this.x = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 0); // col 0 is time
                this.y = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries, 1); // col 1 is value
                this.numpoints = Convert.ToDouble(y.Length);

                // default values for histogram ranges
                this.start = y.Min();
                this.end = y.Max();
                if (start == 0)
                {
                    start = 0.1;
                }
                this.nbin = 10;
                // default values into the text boxes
                textBox_start_value.Text = this.start.ToString();
                textBox_end_value.Text = this.end.ToString();
                textBox_numbins.Text = this.nbin.ToString();
                this.binvalues = get_binvalues(this.start, this.end, this.nbin);
                this.binwidth = binvalues[1] - binvalues[0];



                this.histogram = CStatsFactory.get_histogram(this.y, this.nbin, this.start, this.end);

                double[] normdist = CStatsFactory.get_normal_distribution(binvalues, y);
                double[] gammadist = CStatsFactory.CGammaMethodsFactory.get_gamma_distribution(binvalues, y);

                normdist_scaled = new double[nbin];
                gammadist_scaled = new double[nbin];

                for (int i = 0; i < nbin; i++)
                {
                    normdist_scaled[i] = binwidth * numpoints * normdist[i];
                    gammadist_scaled[i] = binwidth * numpoints * gammadist[i];
                }


                UpdateChartSettings();

                chart_histogram.Series[0].ToolTip = "#VALX, #VALY";

                ////DataView firstView = new DataView(datamaster.Tables[this.selecteddataseries]);

                //// Enable range selection and zooming end user interface
                //chart_histogram.ChartAreas[0].CursorX.IsUserEnabled = true;
                //chart_histogram.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                //chart_histogram.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                //chart_histogram.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
                //chart_histogram.ChartAreas[0].BackColor = Color.Transparent;

                //chart_histogram.Series[0].Points.DataBindXY(binvalues, this.histogram);

                //chart_histogram.Series[0].ChartType = SeriesChartType.Column;

                //this.textBox_numbins.Text = Convert.ToString(this.nbin);
            }
            catch
            {
                MessageBox.Show("This dataseries is empty. Histogram failed to plot");
            }
        }

        private void UpdateChartSettings()
        {

            // Enable range selection and zooming end user interface
            chart_histogram.ChartAreas[0].CursorX.IsUserEnabled = true;
            chart_histogram.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
            chart_histogram.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
            chart_histogram.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
            chart_histogram.ChartAreas[0].BackColor = Color.Transparent;


            chart_histogram.Series["histogram"].Points.DataBindXY(this.binvalues, this.histogram);
            chart_histogram.Series["histogram"].ChartType = SeriesChartType.Column;
            chart_histogram.Series["histogram"]["PointWidth"] = "1.0";
            chart_histogram.Series["histogram"].Color = Color.LightGray;
            chart_histogram.Series["histogram"].BorderColor = Color.Gray;

            chart_histogram.Series["normal"].Points.DataBindXY(this.binvalues, this.normdist_scaled);
            chart_histogram.Series["normal"].ChartType = SeriesChartType.Line;
            chart_histogram.Series["normal"].Color = Color.Blue;

            chart_histogram.Series["gamma"].Points.DataBindXY(this.binvalues, this.gammadist_scaled);
            chart_histogram.Series["gamma"].ChartType = SeriesChartType.Line;
            chart_histogram.Series["gamma"].Color = Color.Red;

            // Set same X axis scale and interval in the single axis data distribution 
            // chart area as in the histogram chart area.
            chart_histogram.ChartAreas[0].AxisX.Minimum = this.start;
            //chart_histogram.ChartAreas[0].AxisX.Minimum = chart_histogram.ChartAreas[0].AxisX.Minimum;
            chart_histogram.ChartAreas[0].AxisX.Maximum = chart_histogram.ChartAreas[0].AxisX.Maximum;
            chart_histogram.ChartAreas[0].AxisX.Interval = chart_histogram.ChartAreas[0].AxisX.Interval;
            chart_histogram.ChartAreas[0].AxisX.RoundAxisValues();
            chart_histogram.ChartAreas[0].AxisX.Title = "Value";
            chart_histogram.ChartAreas[0].AxisY.Title = "Frequency";

            if(plot_normal)
            {
                chart_histogram.Series[1].Enabled = true;
            }
            else
            {
                chart_histogram.Series[1].Enabled = false;
            }
            if (plot_gamma)
            {
                chart_histogram.Series[2].Enabled = true;
            }
            else
            {
                chart_histogram.Series[2].Enabled = false;
            }
        }

        private static double[] get_binvalues(double start, double end, int numinterval)
        {
            double[] x = new double[numinterval];

            double increment = (end - start) / numinterval;
            for (int i = 0; i < numinterval; i++)
            {
                if (i == 0)
                {
                    x[i] = start;
                }
                else
                {
                    x[i] = x[i - 1] + increment;
                }
            }
            return x;

        }

        private void Form3_Load(object sender, EventArgs e)
        {

        }

        private void textBox_numbins_TextChanged(object sender, EventArgs e)
        {
            
            string myStr = textBox_numbins.Text.ToString();

            try
            {
                this.nbin = Convert.ToInt32(myStr);

                //this.histogram = CStatsFactory.gethistogram(this.y, this.nbin, this.start, this.end);
                //UpdateChartSettings();
            }

            catch
            {

            }
        }



        private void textBox_start_value_TextChanged(object sender, EventArgs e)
        {
            string myStr = textBox_start_value.Text.ToString();

            try
            {
                this.start = Convert.ToInt32(myStr);

            }

            catch
            {

            }
        }

        private void textBox_end_value_TextChanged(object sender, EventArgs e)
        {
            string myStr = textBox_end_value.Text.ToString();

            try
            {
                this.end = Convert.ToInt32(myStr);
                //this.histogram = CStatsFactory.gethistogram(this.y, this.nbin, this.start, this.end);
                //UpdateChartSettings();
            }

            catch
            {

            }
        }

        private void button_start_reset_Click(object sender, EventArgs e)
        {
            this.start = this.y.Min();
            textBox_start_value.Text = this.start.ToString();
            //this.histogram = CStatsFactory.gethistogram(this.y, this.nbin, this.start, this.end);
            //UpdateChartSettings();
        }

        private void button_end_reset_Click(object sender, EventArgs e)
        {
            this.end = this.y.Max();
            textBox_end_value.Text = this.end.ToString();
            //this.histogram = CStatsFactory.gethistogram(this.y, this.nbin, this.start, this.end);
            //UpdateChartSettings();
        }

        private void button_recalcuate_Click(object sender, EventArgs e)
        {
            this.binvalues = get_binvalues(this.start, this.end, this.nbin);
            this.histogram = CStatsFactory.get_histogram(this.y, this.nbin, this.start, this.end);

            this.binwidth = binvalues[1] - binvalues[0];

            double[] normdist = CStatsFactory.get_normal_distribution(binvalues, y);
            double[] gammadist = CStatsFactory.CGammaMethodsFactory.get_gamma_distribution(binvalues, y);

            normdist_scaled = new double[nbin];
            gammadist_scaled = new double[nbin];

            for (int i = 0; i < nbin; i++)
            {
                normdist_scaled[i] = this.binwidth * this.numpoints * normdist[i];
                gammadist_scaled[i] = this.binwidth * this.numpoints * gammadist[i];
            }

            UpdateChartSettings();
        }

        private void checkBox_plot_normal_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_plot_normal.Checked == true)
            {
                plot_normal = true;
            }
            else
                plot_normal = false;
            }

        private void checkBox_plot_gamma_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_plot_gamma.Checked == true)
            {
                plot_gamma = true;
            }
            else
            {
                plot_gamma = false;
            }
        
        }

        private void label_numberofbins_Click(object sender, EventArgs e)
        {
        
        }

        private void label_end_value_Click(object sender, EventArgs e)
        {
        
        }

        private void label_start_value_Click(object sender, EventArgs e)
        {
        
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart_histogram.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart_histogram.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp); 
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_histogram.Printing.PrintPreview();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_histogram.Printing.Print(true);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Application.Exit();
            this.Close();
        }

        //private void checkBox_plot_gamma_CheckedChanged(object sender, EventArgs e)
        //{

        //}

        //private void label_numberofbins_Click(object sender, EventArgs e)
        //{

        //}

        //private void label_end_value_Click(object sender, EventArgs e)
        //{

        //}

        //private void label_start_value_Click(object sender, EventArgs e)
        //{

        //}
    }
}