﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_plot_lagged_series : Form
    {
        private Cdatamaster datamaster;
        private string selecteddataseries1;
        private string selecteddataseries2;

        private FormToolkit formtoolkit;

        private double[] x1; // array for holding x axis values
        private double[] y1; // array for holding x axis values

        private double[] x2; // array for holding x axis values
        private double[] y2; // array for holding x axis values

        private double[] x1lagged; // array for holding shifted time series
        private double[] y1lagged; // array for holding shifted time series

        private double[] x2lagged; // array for holding shifted time series
        private double[] y2lagged; // array for holding shifted time series

        private double xmin;
        private double xmax;

        private double ymin;
        private double ymax;

        Series series = new Series();

        public Form_plot_lagged_series(Cdatamaster datamasterToUse, string selecteddataseriesToUse1, string selecteddataseriesToUse2, int lag, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();

            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries1 = selecteddataseriesToUse1;
            this.selecteddataseries2 = selecteddataseriesToUse2;

            this.Text = "Lag plot for "+this.selecteddataseries1+" and "+ this.selecteddataseries2;

            this.formtoolkit = formtoolkitToUse;



            try
            {
                // create series
                this.series = chart1.Series.Add(this.selecteddataseries1);
                this.series = chart1.Series.Add(this.selecteddataseries2);
                this.series = chart1.Series.Add("lagged " + this.selecteddataseries2);

                // assign data to series

                string col0name = Convert.ToString(datamaster.Tables[this.selecteddataseries1].Columns[0].ColumnName);
                string col1name = Convert.ToString(datamaster.Tables[this.selecteddataseries1].Columns[1].ColumnName);

                this.x1 = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries1, 0);
                this.y1 = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries1, 1);

                this.x2 = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries2, 0);
                this.y2 = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries2, 1);

                this.x1lagged = new double[x1.Length];
                this.y1lagged = new double[y1.Length];
                this.x2lagged = new double[x2.Length];
                this.y2lagged = new double[y2.Length];


                if(lag >= 0)
                {
                    //for (int i = lag; i < x1.Length; i++)
                    //{
                    //    this.x1lagged[i-lag] = x1[i];
                    //    this.y1lagged[i-lag] = y1[i];
                    //}
                    for (int i = 0; i < x1.Length; i++)
                    {
                        if (i < x1.Length-lag)
                        {
                            this.x2lagged[i] = x2[i];
                            this.y2lagged[i] = y2[i + lag];
                        }
                        else
                        {
                            this.x2lagged[i] = x2[i];
                            this.y2lagged[i] = double.NaN;
                        }
                    }
                }

                if(lag < 0)
                {
                    for (int i = 0; i < x1.Length; i++)
                    {
                        if (i < Math.Abs(lag))
                        {
                            this.x2lagged[i] = x2[i];
                            this.y2lagged[i] = double.NaN;

                        }
                        else
                        {
                            this.x2lagged[i] = x2[i];
                            this.y2lagged[i] = y2[i - Math.Abs(lag)];
                        }
                    }
                }

                this.xmin = this.x1.Min();
                this.xmin = Math.Round(this.xmin);
                this.xmax = this.x1.Max();
                this.xmax = Math.Round(this.xmax, 0, MidpointRounding.AwayFromZero);

                this.ymin = y1.Min();
                this.ymax = y1.Max();
                if (y2.Min() < this.ymin)
                {
                    this.ymin = y2.Min();
                }
                if (y2lagged.Min() < this.ymin)
                {
                    this.ymin = y2lagged.Min();
                }
                if (y2.Max() > this.ymax)
                {
                    this.ymax = y2.Max();
                }
                if (y2lagged.Max() > this.ymax)
                {
                    this.ymax = y2lagged.Max();
                }


                //this.xmin = this.x1.Min();
                //this.xmax = this.x1.Max();

                // set up default plotting options 

                // Enable range selection and zooming end user interface
                chart1.ChartAreas[0].CursorX.IsUserEnabled = true;
                chart1.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                chart1.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                chart1.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
                chart1.ChartAreas[0].BackColor = Color.Transparent;

                chart1.ChartAreas[0].AxisX.Title = Convert.ToString(col0name);
                chart1.ChartAreas[0].AxisY.Title = Convert.ToString(col1name);

                chart1.ChartAreas[0].AxisX.RoundAxisValues();

                chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
                chart1.ChartAreas[0].AxisX.Maximum = this.xmax;

                chart1.Legends[0].Docking = Docking.Top;
                chart1.Legends[0].Alignment = StringAlignment.Center;
                chart1.Legends[0].Font = new Font("Microsoft Sans Serif", 12);

                chart1.ChartAreas[0].AxisX.IntervalAutoMode = IntervalAutoMode.VariableCount;
                chart1.ChartAreas[0].AxisY.IntervalAutoMode = IntervalAutoMode.VariableCount;

                chart1.ChartAreas[0].AxisX.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisX.TitleFont = new Font("Microsoft Sans Serif", 12);
                chart1.ChartAreas[0].AxisY.TitleFont = new Font("Microsoft Sans Serif", 12);


                UpdateChartSettings();
                chart1.Series[0].ToolTip = "#VALX, #VALY";
                chart1.Series[1].ToolTip = "#VALX, #VALY";
                chart1.Series[2].ToolTip = "#VALX, #VALY";

                chart1.ChartAreas[0].AxisX.LabelStyle.Format = "###.##";
                chart1.ChartAreas[0].AxisY.LabelStyle.Format = "###.##";

            }
            catch
            {
                MessageBox.Show("This dataseries is empty. Please generate the values first");
            }




        }

        public void UpdateChartSettings()
        {
            chart1.Series[this.selecteddataseries1].Points.DataBindXY(this.x1, this.y1);
            chart1.Series[this.selecteddataseries1].ChartType = SeriesChartType.Line;
            chart1.Series[this.selecteddataseries1]["PixelPointWidth"] = "1";
            chart1.Series[this.selecteddataseries1].Color = Color.Black;
            //chart1.Series[this.selecteddataseries1].MarkerStyle = MarkerStyle.Circle;
            //chart1.Series[this.selecteddataseries1].MarkerSize = 5;
            //chart1.Series[this.selecteddataseries1].MarkerColor = Color.White;
            //chart1.Series[this.selecteddataseries1].MarkerBorderColor = Color.Black;
            //chart1.Series[this.selecteddataseries1].MarkerBorderWidth = 1;

            chart1.Series[this.selecteddataseries2].Points.DataBindXY(this.x2, this.y2);
            chart1.Series[this.selecteddataseries2].ChartType = SeriesChartType.Line;
            chart1.Series[this.selecteddataseries2]["PixelPointWidth"] = "1";
            chart1.Series[this.selecteddataseries2].Color = Color.Gray;
            //chart1.Series[this.selecteddataseries2].MarkerStyle = MarkerStyle.Circle;
            //chart1.Series[this.selecteddataseries2].MarkerSize = 5;
            //chart1.Series[this.selecteddataseries2].MarkerColor = Color.White;
            //chart1.Series[this.selecteddataseries2].MarkerBorderColor = Color.Gray;
            //chart1.Series[this.selecteddataseries2].MarkerBorderWidth = 1;

            chart1.Series[2].Points.DataBindXY(this.x2lagged, this.y2lagged);
            chart1.Series[2].ChartType = SeriesChartType.Line;
            chart1.Series[2]["PixelPointWidth"] = "1";
            chart1.Series[2].Color = Color.Red;
            chart1.Series[2].BorderDashStyle = ChartDashStyle.Dash;
            chart1.Series[2].EmptyPointStyle.Color = Color.Red;
            chart1.Series[2].EmptyPointStyle.BorderWidth = 0;
            chart1.Series[2].EmptyPointStyle.MarkerStyle = MarkerStyle.None;

            chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
            chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = false;

            chart1.ChartAreas[0].AxisX.Minimum = this.xmin;
            chart1.ChartAreas[0].AxisX.Maximum = this.xmax;
            chart1.ChartAreas[0].AxisY.Minimum = this.ymin;
            chart1.ChartAreas[0].AxisY.Maximum = this.ymax;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart1.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart1.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp); 
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Printing.PrintPreview();
        }

        private void printToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            chart1.Printing.Print(true);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        public double get_xmin
        {
            set { this.xmin = value; }
        }

        public double get_xmax
        {
            set { this.xmax = value; }
        }

        public double get_ymin
        {
            set { this.ymin = value; }
        }

        public double get_ymax
        {
            set { this.ymax = value; }
        }

        private void axisPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form_plot_axis_properties form_plot_axis_properties = new Form_plot_axis_properties(this.xmin, this.xmax, this.ymin, this.ymax, this);
            form_plot_axis_properties.Show();
        }
    }
}
