﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Climate_toolkit
{
    public partial class Form_plot_regression : Form
    {
        private Cdatamaster datamaster;
        private FormToolkit formtoolkit;

        private string selecteddataseries1;
        private string selecteddataseries2;
        private string selectedtable;
        private string scatterplot;
        private string regressionline;

        private double[] x_scatter; // array for holding x scatter values
        private double[] y_scatter; // array for holding y scatter values

        private double[] x_line = new double[2]; // array for holding x regression line
        private double[] y_line = new double[2]; // array for holding y regression line

        private double xmin;
        private double xmax;
        private double y_xmin;
        private double y_xmax;

        private double a0;
        private double a1;
        private double[] tableinfo;

        Series series = new Series();

        public Form_plot_regression(Cdatamaster datamasterToUse, string selecteddataseriesToUse1, string selecteddataseriesToUse2, string selectedtableToUse, FormToolkit formtoolkitToUse)
        {
            InitializeComponent();

            this.datamaster = datamasterToUse; // make a local copy this subform holds on to
            this.selecteddataseries1 = selecteddataseriesToUse1;
            this.selecteddataseries2 = selecteddataseriesToUse2;
            this.selectedtable = selectedtableToUse;
            this.scatterplot = "scatter plot";
            this.regressionline = "regression line";
            this.formtoolkit = formtoolkitToUse;

            // assign data to series
            this.series = chart_regression.Series.Add(scatterplot);
            x_scatter = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries1, 1);
            y_scatter = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selecteddataseries2, 1);

            tableinfo = this.formtoolkit.get_oneColArray_from_datatable(datamaster, selectedtable, 1);
            a0 = tableinfo[1];
            a1 = tableinfo[0];

            this.series = chart_regression.Series.Add(regressionline);

            CRegressionFactory.get_regression_line_onepredictor(x_scatter, a0, a1, out this.xmin, out this.xmax, out this.y_xmin, out this.y_xmax);

            this.x_line[0] = xmin;
            this.x_line[1] = xmax;
            y_line[0] = y_xmin;
            y_line[1] = y_xmax;

            // set up default plotting options 
            chart_regression.ChartAreas[0].AxisX.Title = selecteddataseries1;
            chart_regression.ChartAreas[0].AxisY.Title = selecteddataseries2;

            chart_regression.ChartAreas[0].CursorX.IsUserEnabled = true;
            chart_regression.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
            chart_regression.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
            chart_regression.ChartAreas[0].AxisX.ScrollBar.IsPositionedInside = true;
            chart_regression.ChartAreas[0].BackColor = Color.Transparent;

            chart_regression.Legends[0].Docking = Docking.Top;
            chart_regression.Legends[0].Alignment = StringAlignment.Center;

            chart_regression.ChartAreas[0].AxisX.IntervalAutoMode = IntervalAutoMode.VariableCount;
            chart_regression.ChartAreas[0].AxisY.IntervalAutoMode = IntervalAutoMode.VariableCount;

            chart_regression.ChartAreas[0].AxisX.LabelStyle.Format = "###.##";
            chart_regression.ChartAreas[0].AxisY.LabelStyle.Format = "###.##";




            chart_regression.Series[0].ToolTip = "#VALX, #VALY";
            chart_regression.Series[1].ToolTip = "#VALX, #VALY";
            //chart_regression.Series[2].ToolTip = "#VALX, #VALY";

            chart_regression.ChartAreas[0].AxisX.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
            chart_regression.ChartAreas[0].AxisY.LabelStyle.Font = new Font("Microsoft Sans Serif", 12);
            chart_regression.ChartAreas[0].AxisX.TitleFont = new Font("Microsoft Sans Serif", 12);
            chart_regression.ChartAreas[0].AxisY.TitleFont = new Font("Microsoft Sans Serif", 12);

            UpdateChartSettings();

        }

        private void UpdateChartSettings()
        {
            chart_regression.Series[this.scatterplot].Points.DataBindXY(this.x_scatter, this.y_scatter);
            chart_regression.Series[this.scatterplot].ChartType = SeriesChartType.Point;
            chart_regression.Series[this.scatterplot].Color = Color.Black;

            chart_regression.Series[this.regressionline].Points.DataBindXY(this.x_line, this.y_line);
            chart_regression.Series[this.regressionline].ChartType = SeriesChartType.Line;
            chart_regression.Series[this.regressionline].Color = Color.Black;
            chart_regression.Series[this.regressionline].Enabled = true;

            chart_regression.ChartAreas[0].AxisX.RoundAxisValues();
            chart_regression.ChartAreas[0].AxisX.MajorGrid.Enabled = false;
            chart_regression.ChartAreas[0].AxisY.MajorGrid.Enabled = false;
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {

        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_regression.Printing.Print(true);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|SVG (*.svg)|*.svg|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("emf"))
                {
                    format = ChartImageFormat.Emf;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }
                else if (saveFileDialog1.FileName.EndsWith("tif"))
                {
                    format = ChartImageFormat.Tiff;
                }
                //else if (saveFileDialog1.FileName.EndsWith("svg"))
                //{
                //    format = ChartImageFormat.Svg;
                //}

                // Save image
                chart_regression.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart_regression.Printing.PrintPreview();
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // create a memory stream to save the chart image    
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            // save the chart image to the stream    
            chart_regression.SaveImage(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            // create a bitmap using the stream    
            Bitmap bmp = new Bitmap(stream);

            // save the bitmap to the clipboard    
            Clipboard.SetDataObject(bmp);
        }
    }
}
